! ================================================================================================================================
!  MODULE       : explicitsnow
!
!  CONTACT      : orchidee-help _at_ listes.ipsl.fr
!
!  LICENCE      : IPSL (2006)
!  This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF  Computes hydrologic snow processes on continental points.
!!
!!\n DESCRIPTION: This module computes hydrologic snow processes on continental points.
!!
!! RECENT CHANGES: 
!!
!! REFERENCES	: None
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE explicitsnow
  USE ioipsl_para
  ! modules used :
  USE constantes_var
  USE constantes_soil
  USE constantes
  USE time, ONLY : one_day, dt_sechiba
  USE pft_parameters 
  USE qsat_moisture 
  USE sechiba_io_p
  USE xios_orchidee

  IMPLICIT NONE

  ! Public routines :
  PRIVATE
  PUBLIC :: explicitsnow_main, explicitsnow_initialize, explicitsnow_finalize

  LOGICAL, SAVE                     :: writeforcheck_explicitsnow=.FALSE.           !! switch for write to check by yx !yx:+-
!$OMP THREADPRIVATE(writeforcheck_explicitsnow)

CONTAINS

!================================================================================================================================
!! SUBROUTINE   : explicitsnow_initialize
!!
!>\BRIEF        Read variables for explictsnow module from restart file
!!                
!! DESCRIPTION  : Read variables for explictsnow module from restart file
!!
!! \n
!_
!================================================================================================================================
  SUBROUTINE explicitsnow_initialize( kjit,     kjpindex, rest_id,    snowrho,   &
                                      snowtemp, snowdz,   snowheat,   snowgrain, &
                                      snowrho_pft, snowtemp_pft, snowdz_pft, snowheat_pft, snowgrain_pft) !yx:+-

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                           :: kjit           !! Time step number 
    INTEGER(i_std), INTENT(in)                           :: kjpindex       !! Domain size
    INTEGER(i_std),INTENT (in)                           :: rest_id        !! Restart file identifier
    
    !! 0.2 Output variables
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(out)  :: snowrho        !! Snow density (Kg/m^3)
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(out)  :: snowtemp       !! Snow temperature
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(out)  :: snowdz         !! Snow layer thickness
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(out)  :: snowheat       !! Snow heat content (J/m^2)
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(out)  :: snowgrain      !! Snow grainsize (m)
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(out)  :: snowrho_pft        !! Snow density (Kg/m^3) !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(out)  :: snowtemp_pft       !! Snow temperature !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(out)  :: snowdz_pft         !! Snow layer thickness !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(out)  :: snowheat_pft       !! Snow heat content (J/m^2) !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(out)  :: snowgrain_pft      !! Snow grainsize (m) !yx:+-
    
    
    !! 1. Read from restart file
    CALL restget_p (rest_id, 'snowrho', nbp_glo, nsnow, 1, kjit,.TRUE.,snowrho, "gather", nbp_glo, index_g)
    CALL setvar_p (snowrho, val_exp, 'Snow Density profile', xrhosmin)
    CALL restget_p (rest_id, 'snowrho_pft', nbp_glo, nsnow, nvm, kjit,.TRUE.,snowrho_pft, "gather", nbp_glo, index_g) !yx:+-
    CALL setvar_p (snowrho_pft, val_exp, 'Snow Density profile (pft)', xrhosmin) !yx:+-
    
    CALL restget_p (rest_id, 'snowtemp', nbp_glo,  nsnow, 1, kjit,.TRUE.,snowtemp, "gather", nbp_glo,  index_g)
    CALL setvar_p (snowtemp, val_exp, 'Snow Temperature profile', tp_00)
    CALL restget_p (rest_id, 'snowtemp_pft', nbp_glo,  nsnow, nvm, kjit,.TRUE.,snowtemp_pft, "gather", nbp_glo,  index_g) !yx:+-
    CALL setvar_p (snowtemp_pft, val_exp, 'Snow Temperature profile (pft)', tp_00) !yx:+-
    
    CALL restget_p (rest_id, 'snowdz', nbp_glo,  nsnow, 1, kjit,.TRUE.,snowdz, "gather", nbp_glo,  index_g)
    CALL setvar_p (snowdz, val_exp, 'Snow depth profile', 0.0)
    CALL restget_p (rest_id, 'snowdz_pft', nbp_glo,  nsnow, nvm, kjit,.TRUE.,snowdz_pft, "gather", nbp_glo,  index_g) !yx:+-
    CALL setvar_p (snowdz_pft, val_exp, 'Snow depth profile (pft)', 0.0) !yx:+-
    
    CALL restget_p (rest_id, 'snowheat', nbp_glo,  nsnow, 1, kjit,.TRUE.,snowheat, "gather", nbp_glo,  index_g)
    CALL setvar_p (snowheat, val_exp, 'Snow Heat profile', 0.0)
    CALL restget_p (rest_id, 'snowheat_pft', nbp_glo,  nsnow, nvm, kjit,.TRUE.,snowheat_pft, "gather", nbp_glo,  index_g) !yx:+-
    CALL setvar_p (snowheat_pft, val_exp, 'Snow Heat profile (pft)', 0.0) !yx:+-
    
    CALL restget_p (rest_id, 'snowgrain', nbp_glo,  nsnow, 1, kjit,.TRUE.,snowgrain, "gather", nbp_glo,  index_g)
    CALL setvar_p (snowgrain, val_exp, 'Snow grain profile', 0.0)
    CALL restget_p (rest_id, 'snowgrain_pft', nbp_glo,  nsnow, nvm, kjit,.TRUE.,snowgrain_pft, "gather", nbp_glo,  index_g) !yx:+-
    CALL setvar_p (snowgrain_pft, val_exp, 'Snow grain profile (pft)', 0.0) !yx:+-
    
  END SUBROUTINE explicitsnow_initialize
  

!================================================================================================================================
!! SUBROUTINE   : explicitsnow_main
!!
!>\BRIEF        Main call for snow calculations
!!                
!! DESCRIPTION  : Main routine for calculation of the snow processes with explictsnow module.
!!
!! RECENT CHANGE(S) : None 
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : 
!!
!! FLOWCHART    : None
!! \n
!_
!================================================================================================================================
 SUBROUTINE explicitsnow_main(kjpindex,     precip_rain,   precip_snow,    temp_air,    pb,           &
                              u,            v,             temp_sol_new,   soilcap,     pgflux,       &
                              frac_nobio,   totfrac_nobio, gtemp,                                     &
                              lambda_snow,  cgrnd_snow,    dgrnd_snow,                                & ! in  
                              vevapsno,     snow_age,      snow_nobio_age, snow_nobio,  snowrho,      &
                              snowgrain,    snowdz,        snowtemp,       snowheat,    snow,         &
                              temp_sol_add,                                                           &
                              snowliq,      subsnownobio, grndflux,      snowmelt,       tot_melt,    &
                              soilflxresid, subsinksoil, &
                              soilcap_pft, pgflux_pft, gtemp_pft, soilflxresid_pft, lambda_snow_pft, cgrnd_snow_pft, & !yx:+-
                              dgrnd_snow_pft, temp_sol_new_pft, snowmelt_pft, tot_melt_pft, vevapsno_pft, subsinksoil_pft, & !yx:+-
                              snow_age_pft, snowtemp_pft, snowrho_pft, snowgrain_pft, & !yx:+-
                              snowdz_pft, snowheat_pft, snowliq_pft, snow_pft, grndflux_pft, temp_sol_add_pft, veget_max) !yx:+-

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                               :: kjpindex         !! Domain size
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: precip_rain      !! Rainfall
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: precip_snow      !! Snowfall
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: temp_air         !! Air temperature
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: pb               !! Surface pressure
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: u,v              !! Horizontal wind speed

    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: soilcap          !! Soil capacity
    REAL(r_std),DIMENSION (kjpindex,nvm),INTENT(in)          :: soilcap_pft      !! Soil heat capacity !yx:+-
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: pgflux           !! Net energy into snowpack
    REAL(r_std),DIMENSION (kjpindex,nvm),INTENT(in)          :: pgflux_pft       !! Net energy into snowpack(w/m2) !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nnobio), INTENT(in)     :: frac_nobio       !! Fraction of continental ice, lakes, ...
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: totfrac_nobio    !! Total fraction of continental ice+lakes+ ...
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: gtemp            !! First soil layer temperature
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(in)        :: gtemp_pft        !! First soil layer temperature !yx:+-
    !TW
    REAL(r_std), DIMENSION (kjpindex),INTENT(inout)          :: soilflxresid
    REAL(r_std), DIMENSION (kjpindex,nvm),INTENT(inout)      :: soilflxresid_pft !yx:+-
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: lambda_snow      !! Coefficient of the linear extrapolation of surface temperature 
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(in)        :: lambda_snow_pft  !! Coefficient of the linear extrapolation of surface temperature !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT (in)     :: cgrnd_snow       !! Integration coefficient for snow numerical scheme
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT (in) :: cgrnd_snow_pft   !! Integration coefficient for snow numerical scheme !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT (in)     :: dgrnd_snow       !! Integration coefficient for snow numerical scheme
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT (in) :: dgrnd_snow_pft   !! Integration coefficient for snow numerical scheme !yx:+-
    REAL(r_std), DIMENSION (kjpindex), INTENT (in)           :: temp_sol_new     !! skin temperature
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)       :: temp_sol_new_pft !! skin temperature !yx:+-
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)        :: veget_max        !! Max. fraction of vegetation type (LAI -> infty)


    !! 0.2 Output fields
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: snowmelt         !! Snow melt [mm/dt_sechiba]
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(inout)     :: snowmelt_pft     !! Snow melt [mm/dt_sechiba] !yx:+-
    REAL(r_std), DIMENSION (kjpindex), INTENT(out)           :: tot_melt         !! Total melt from ice and snow
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(out)       :: tot_melt_pft     !! Total melt from ice and snow !yx:+-




    !! 0.3 Modified fields
    REAL(r_std), DIMENSION (kjpindex,nnobio), INTENT(inout)  :: subsnownobio     !! Sublimation of snow on other surface types (ice, lakes, ...)
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: vevapsno         !! Snow evaporation  @tex ($kg m^{-2}$) @endtex 
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(inout)     :: vevapsno_pft     !! Snow evaporation  @tex ($kg m^{-2}$) @endtex !yx:+-
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: subsinksoil      !! Excess of sublimation as a sink for the soil
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(inout)     :: subsinksoil_pft  !! Excess of sublimation as a sink for the soil !yx:+-
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: snow_age         !! Snow age
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(inout)     :: snow_age_pft     !! Snow age !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nnobio), INTENT(inout)  :: snow_nobio       !! Ice water balance
    REAL(r_std), DIMENSION (kjpindex,nnobio), INTENT(inout)  :: snow_nobio_age   !! Snow age on ice, lakes, ...
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(inout)   :: snowtemp         !! Snow temperature
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(inout)   :: snowtemp_pft     !! Snow temperature !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(inout)   :: snowrho          !! Snow density (Kg/m^3)
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(inout)   :: snowrho_pft      !! Snow density (Kg/m^3) !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(inout)   :: snowgrain        !! Snow grainsize (m)
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(inout)   :: snowgrain_pft    !! Snow grainsize (m) !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(inout)   :: snowdz           !! Snow layer thickness [m]
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(inout)   :: snowdz_pft       !! Snow layer thickness [m] !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(inout)   :: snowheat         !! Snow heat content (J/m^2)
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(inout)   :: snowheat_pft     !! Snow heat content (J/m^2) !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(inout)   :: snowliq          !! Snow liquid content (m)
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(inout)   :: snowliq_pft      !! Snow liquid content (m) !yx:+-
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: snow             !! Snow mass [Kg/m^2]
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(inout)         :: snow_pft         !! Snow mass [Kg/m^2] !yx:+-
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: grndflux         !! Net flux into soil [W/m2]
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(inout)         :: grndflux_pft     !! Net flux into soil [W/m2] !yx:+-
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: temp_sol_add     !! Additional energy to melt snow for snow ablation case (K)
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(inout)         :: temp_sol_add_pft !! Additional energy to melt snow for snow ablation case (K) !yx:+-

 
    !! 0.4 Local declaration
    INTEGER(i_std)                                           :: ji, iv, jj,m,jv
    REAL(r_std),DIMENSION  (kjpindex)                        :: snow_depth_tmp
    REAL(r_std),DIMENSION  (kjpindex,nvm)                    :: snow_depth_tmp_pft !yx:+-
    REAL(r_std),DIMENSION  (kjpindex)                        :: snowmelt_from_maxmass
    REAL(r_std),DIMENSION  (kjpindex,nvm)                    :: snowmelt_from_maxmass_pft !yx:+-
    REAL(r_std)                                              :: snowdzm1
    REAL(r_std), DIMENSION (kjpindex)                        :: thrufal          !! Water leaving snow pack [kg/m2/s]
    REAL(r_std), DIMENSION (kjpindex,nvm)                    :: thrufal_pft      !! Water leaving snow pack [kg/m2/s] !yx:+-
    REAL(r_std), DIMENSION (kjpindex)                        :: d_age            !! Snow age change
    REAL(r_std), DIMENSION (kjpindex,nvm)                    :: d_age_pft        !! Snow age change !yx:+-
    REAL(r_std), DIMENSION (kjpindex)                        :: xx               !! Temporary
    REAL(r_std), DIMENSION (kjpindex,nvm)                    :: xx_pft           !! Temporary !yx:+-
    REAL(r_std), DIMENSION (kjpindex)                        :: snowmelt_tmp,snowmelt_ice,icemelt,temp_sol_new_old
    REAL(r_std), DIMENSION (kjpindex,nsnow)                  :: snowdz_old
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm)              :: snowdz_old_pft !yx:+-
    REAL(r_std), DIMENSION (kjpindex)                        :: ZLIQHEATXS
    REAL(r_std), DIMENSION (kjpindex,nvm)                    :: ZLIQHEATXS_pft !yx:+-
    REAL(r_std), DIMENSION (kjpindex)                        :: ZSNOWEVAPS, ZSNOWDZ,subsnowveg,ZSNOWMELT_XS
    REAL(r_std), DIMENSION (kjpindex,nvm)                    :: ZSNOWEVAPS_pft, ZSNOWDZ_pft,subsnowveg_pft,ZSNOWMELT_XS_pft !yx:+-
    REAL(r_std)                                              :: maxmass_snowdepth, snow_d1k
    REAL(r_std), DIMENSION (nvm)                             :: maxmass_snowdepth_pft, snow_d1k_pft !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow)                  :: WSNOWDZ,SMASS
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm)              :: WSNOWDZ_pft,SMASS_pft !yx:+-
    REAL(r_std), DIMENSION (kjpindex)                        :: SMASSC,snowacc,snow_remove
    REAL(r_std), DIMENSION (kjpindex,nvm)                    :: SMASSC_pft,snowacc_pft,snow_remove_pft !yx:+-
    INTEGER(i_std) :: locjj
    REAL(r_std)                                              :: grndflux_tmp
    REAL(r_std), DIMENSION (nvm)                             :: grndflux_tmp_pft !yx:+-
    REAL(r_std), DIMENSION (nsnow)                           :: snowtemp_tmp
    REAL(r_std), DIMENSION (nsnow,nvm)                       :: snowtemp_tmp_pft !yx:+-
    
    REAL(r_std)                                              :: s2flux_tmp,fromsoilflux
    REAL(r_std), DIMENSION (nvm)                             :: s2flux_tmp_pft,fromsoilflux_pft !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow)                  :: pcapa_snow 
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm)              :: pcapa_snow_pft !yx:+-
    REAL(r_std), DIMENSION (kjpindex)                        :: psnowhmass
    REAL(r_std), DIMENSION (kjpindex,nvm)                    :: psnowhmass_pft !yx:+-
    REAL(r_std), PARAMETER                                   :: XP00 = 1.E5


    !! 1. Initialization
    
    temp_sol_new_old = temp_sol_new
    snowmelt_ice(:) = zero
    icemelt(:) = zero
    tot_melt(:) = zero
    snowmelt(:) = zero
    snowmelt_from_maxmass(:) = zero

    tot_melt_pft(:,:) = zero !yx:+-
    snowmelt_pft(:,:) = zero !yx:+-
    snowmelt_from_maxmass_pft(:,:) = zero !yx:+-
    
    !! 2. on Vegetation
    ! 2.1 Snow fall
    CALL explicitsnow_fall(kjpindex,precip_snow,temp_air,u,v,totfrac_nobio,snowrho,snowdz,&
         & snowheat,snowgrain,snowtemp,psnowhmass,&
         & snowtemp_pft, psnowhmass_pft, snowrho_pft, snowdz_pft, snowheat_pft, snowgrain_pft) !yx:+-

    IF (writeforcheck_explicitsnow) THEN
       WRITE(numout,*) 'after explicitsnow_fall'
       WRITE(numout,*) 'yi debug: snowtemp(1,:)=', snowtemp(1,:)
       WRITE(numout,*) 'yi debug: snowtemp_pft(1,:,:)=', snowtemp_pft(1,:,:)
       WRITE(numout,*) 'yi debug: psnowhmass(1)=', psnowhmass(1)
       WRITE(numout,*) 'yi debug: psnowhmass_pft(1,:)=', psnowhmass_pft(1,:)
       WRITE(numout,*) 'yi debug: snowrho(1,:)=', snowrho(1,:)
       WRITE(numout,*) 'yi debug: snowrho_pft(1,:,:)=', snowrho_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowdz(1,:)=', snowdz(1,:)
       WRITE(numout,*) 'yi debug: snowdz_pft(1,:,:)=', snowdz_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowgrain(1,:)=', snowgrain(1,:)
       WRITE(numout,*) 'yi debug: snowgrain_pft(1,:,:)=', snowgrain_pft(1,:,:)
    ENDIF
    
    ! 2.2 calculate the new snow discretization
    snow_depth_tmp(:) = SUM(snowdz(:,:),2)
    snow_depth_tmp_pft(:,:) = SUM(snowdz_pft(:,:,:),2) !yx:+-
    snowdz_old = snowdz
    snowdz_old_pft = snowdz_pft !yx:+-
    
    CALL explicitsnow_levels(kjpindex,snow_depth_tmp, snowdz, &
         & snow_depth_tmp_pft, snowdz_pft) !yx:+-

    IF (writeforcheck_explicitsnow) THEN
       WRITE(numout,*) 'after explicitsnow_levels'
       WRITE(numout,*) 'yi debug: snowdz(1,:)=', snowdz(1,:)
       WRITE(numout,*) 'yi debug: snowdz_pft(1,:,:)=', snowdz_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snow_depth_tmp(1)=', snow_depth_tmp(1)
       WRITE(numout,*) 'yi debug: snow_depth_tmp_pft(1,:)=', snow_depth_tmp_pft(1,:)
    ENDIF
    
    ! 2.3 Snow heat redistribution
    CALL explicitsnow_transf(kjpindex,snowdz_old,snowdz,snowrho,snowheat,snowgrain,&
         snowdz_old_pft, snowrho_pft, snowgrain_pft, snowdz_pft, snowheat_pft) !yx:+-

    IF (writeforcheck_explicitsnow) THEN
       WRITE(numout,*) 'after explicitsnow_transf'
       WRITE(numout,*) 'yi debug: snowdz(1,:)=', snowdz(1,:)
       WRITE(numout,*) 'yi debug: snowdz_pft(1,:,:)=', snowdz_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowdz_old(1,:)=', snowdz_old(1,:)
       WRITE(numout,*) 'yi debug: snowdz_old_pft(1,:,:)=', snowdz_old_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowrho(1,:)=', snowrho(1,:)
       WRITE(numout,*) 'yi debug: snowrho_pft(1,:,:)=', snowrho_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowgrain(1,:)=', snowgrain(1,:)
       WRITE(numout,*) 'yi debug: snowgrain_pft(1,:,:)=', snowgrain_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowheat(1,:)=', snowheat(1,:)
       WRITE(numout,*) 'yi debug: snowheat_pft(1,:,:)=', snowheat_pft(1,:,:)
    ENDIF

    
    ! 2.4 Diagonize water portion of the snow from snow heat content:
    DO ji=1, kjpindex
       IF (SUM(snowdz(ji,:)) .GT. 0.0) THEN
          snowtemp(ji,:) = snow3ltemp_1d(snowheat(ji,:),snowrho(ji,:),snowdz(ji,:))
          snowliq(ji,:) = snow3lliq_1d(snowheat(ji,:),snowrho(ji,:),snowdz(ji,:),snowtemp(ji,:))
       ELSE
          snowliq(ji,:) = zero
          snowtemp(ji,:) = tp_00
       ENDIF
    END DO

   !yx:+++++
    DO ji=1, kjpindex
       DO jv=1,nvm
          IF (SUM(snowdz_pft(ji,:,jv)) .GT. 0.0) THEN
             snowtemp_pft(ji,:,jv) = snow3ltemp_1d(snowheat_pft(ji,:,jv),snowrho_pft(ji,:,jv),snowdz_pft(ji,:,jv))
             snowliq_pft(ji,:,jv) = snow3lliq_1d(snowheat_pft(ji,:,jv),snowrho_pft(ji,:,jv),snowdz_pft(ji,:,jv),snowtemp_pft(ji,:,jv))
          ELSE
             snowliq_pft(ji,:,jv) = zero
             snowtemp_pft(ji,:,jv) = tp_00
          ENDIF
       ENDDO
    END DO
   !yx:-----

    IF (writeforcheck_explicitsnow) THEN
       WRITE(numout,*) 'after 2.4'
       WRITE(numout,*) 'yi debug: snowtemp(1,:)=', snowtemp(1,:)
       WRITE(numout,*) 'yi debug: snowtemp_pft(1,:,:)=', snowtemp_pft(1,:,:)
    ENDIF
    
    ! 2.5 snow compaction
    CALL explicitsnow_compactn(kjpindex,snowtemp,snowrho,snowdz,&
         snowtemp_pft, snowrho_pft, snowdz_pft) !yx:+-
    ! Update snow heat 
    DO ji = 1, kjpindex
       snowheat(ji,:) = snow3lheat_1d(snowliq(ji,:),snowrho(ji,:),snowdz(ji,:),snowtemp(ji,:))
       !yx:+++++
       DO jv=1,nvm
          snowheat_pft(ji,:,jv) = snow3lheat_1d(snowliq_pft(ji,:,jv),snowrho_pft(ji,:,jv),snowdz_pft(ji,:,jv),snowtemp_pft(ji,:,jv))
       ENDDO
       !yx:-----
    ENDDO

    IF (writeforcheck_explicitsnow) THEN
       WRITE(numout,*) 'after explicitsnow_compactn'
       WRITE(numout,*) 'yi debug: snowtemp(1,:)=', snowtemp(1,:)
       WRITE(numout,*) 'yi debug: snowtemp_pft(1,:,:)=', snowtemp_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowrho(1,:)=', snowrho(1,:)
       WRITE(numout,*) 'yi debug: snowrho_pft(1,:,:)=', snowrho_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowdz(1,:)=', snowdz(1,:)
       WRITE(numout,*) 'yi debug: snowdz_pft(1,:,:)=', snowdz_pft(1,:,:)
    ENDIF

    ! 2.6 Calculate the snow temperature profile based on heat diffusion
    CALL explicitsnow_profile (kjpindex,cgrnd_snow,dgrnd_snow,lambda_snow,temp_sol_new, snowtemp,snowdz,temp_sol_add,&
         temp_sol_new_pft,cgrnd_snow_pft,dgrnd_snow_pft,lambda_snow_pft,snowdz_pft,temp_sol_add_pft,snowtemp_pft)!yx:+-

    IF (writeforcheck_explicitsnow) THEN
       WRITE(numout,*) 'after explicitsnow_profile'
       WRITE(numout,*) 'yi debug: snowtemp(1,:)=', snowtemp(1,:)
       WRITE(numout,*) 'yi debug: snowtemp_pft(1,:,:)=', snowtemp_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowdz(1,:)=', snowdz(1,:)
       WRITE(numout,*) 'yi debug: snowdz_pft(1,:,:)=', snowdz_pft(1,:,:)
    ENDIF
    
    ! 2.7 Test whether snow is existed on the ground or not
    grndflux(:)=0.0
    grndflux_pft(:,:)=0.0 !yx:+-
    CALL explicitsnow_gone(kjpindex,pgflux,&
         & snowheat,snowtemp,snowdz,snowrho,snowliq,grndflux,snowmelt,soilflxresid,&
         & pgflux_pft, snowheat_pft, soilflxresid_pft, snowtemp_pft, snowdz_pft, snowrho_pft, & !yx:+-
         & snowliq_pft, grndflux_pft, snowmelt_pft, thrufal_pft) !yx:+-

    IF (writeforcheck_explicitsnow) THEN
       WRITE(numout,*) 'after explicitsnow_gone'
       WRITE(numout,*) 'yi debug: snowtemp(1,:)=', snowtemp(1,:)
       WRITE(numout,*) 'yi debug: snowtemp_pft(1,:,:)=', snowtemp_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowdz(1,:)=', snowdz(1,:)
       WRITE(numout,*) 'yi debug: snowdz_pft(1,:,:)=', snowdz_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowrho(1,:)=', snowrho(1,:)
       WRITE(numout,*) 'yi debug: snowrho_pft(1,:,:)=', snowrho_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowheat(1,:)=', snowheat(1,:)
       WRITE(numout,*) 'yi debug: snowheat_pft(1,:,:)=', snowheat_pft(1,:,:)
       WRITE(numout,*) 'yi debug: pgflux(1)=', pgflux(1)
       WRITE(numout,*) 'yi debug: pgflux_pft(1,:)=', pgflux_pft(1,:)
       WRITE(numout,*) 'yi debug: soilflxresid(1)=', soilflxresid(1)
       WRITE(numout,*) 'yi debug: soilflxresid_pft(1,:)=', soilflxresid_pft(1,:)
       WRITE(numout,*) 'yi debug: snowliq(1,:)=', snowliq(1,:)
       WRITE(numout,*) 'yi debug: snowliq_pft(1,:,:)=', snowliq_pft(1,:,:)
       WRITE(numout,*) 'yi debug: grndflux(1)=', grndflux(1)
       WRITE(numout,*) 'yi debug: grndflux_pft(1,:)=', grndflux_pft(1,:)
       WRITE(numout,*) 'yi debug: snowmelt(1)=', snowmelt(1)
       WRITE(numout,*) 'yi debug: snowmelt_pft(1,:)=', snowmelt_pft(1,:)
       WRITE(numout,*) 'yi debug: thrufal(1)=', thrufal(1)
       WRITE(numout,*) 'yi debug: thrufal_pft(1,:)=', thrufal_pft(1,:)
    ENDIF

    ! 2.8 Calculate snow melt/refreezing processes
    CALL explicitsnow_melt_refrz(kjpindex,precip_rain,pgflux,soilcap,&
         & snowtemp,snowdz,snowrho,snowliq,snowmelt,grndflux,temp_air,soilflxresid,&
         & pcapa_snow_pft, pgflux_pft, soilcap_pft, soilflxresid_pft, & !yx:+-
         & snowtemp_pft, snowdz_pft, snowrho_pft, snowliq_pft, grndflux_pft, snowmelt_pft) !yx:+-

    IF (writeforcheck_explicitsnow) THEN
       WRITE(numout,*) 'after explicitsnow_melt_refrz'
       WRITE(numout,*) 'yi debug: snowtemp(1,:)=', snowtemp(1,:)
       WRITE(numout,*) 'yi debug: snowtemp_pft(1,:,:)=', snowtemp_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowdz(1,:)=', snowdz(1,:)
       WRITE(numout,*) 'yi debug: snowdz_pft(1,:,:)=', snowdz_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowrho(1,:)=', snowrho(1,:)
       WRITE(numout,*) 'yi debug: snowrho_pft(1,:,:)=', snowrho_pft(1,:,:)
       WRITE(numout,*) 'yi debug: snowliq(1,:)=', snowliq(1,:)
       WRITE(numout,*) 'yi debug: snowliq_pft(1,:,:)=', snowliq_pft(1,:,:)
       WRITE(numout,*) 'yi debug: grndflux(1)=', grndflux(1)
       WRITE(numout,*) 'yi debug: grndflux_pft(1,:)=', grndflux_pft(1,:)
       WRITE(numout,*) 'yi debug: snowmelt(1)=', snowmelt(1)
       WRITE(numout,*) 'yi debug: snowmelt_pft(1,:)=', snowmelt_pft(1,:)
    ENDIF

    ! 2.9 Snow sublimation changing snow thickness
    snow(:) = 0.0
    snow_pft(:,:) = 0.0
    DO ji=1,kjpindex !domain size
       snow(ji) = SUM(snowrho(ji,:) * snowdz(ji,:))
       !yx:+++++
       DO jv=1,nvm
          snow_pft(ji,jv) = SUM(snowrho_pft(ji,:,jv) * snowdz_pft(ji,:,jv))
       ENDDO
       !yx:-----
    ENDDO
    
    subsnownobio(:,:) = zero
    subsinksoil(:) = zero
    subsinksoil_pft(:,:) = zero !yx:+-
    
    DO ji=1, kjpindex ! domain size
       IF ( snow(ji) > snowcri ) THEN
          ! There is enough snow to split the sublimation demand between the nobio and vegetation parts.
          IF (snow_nobio(ji,iice) .GE. vevapsno(ji)) THEN
             ! There is enough snow on the ice fraction to sustain the sublimation demand.
             subsnownobio(ji,iice) = frac_nobio(ji,iice)*vevapsno(ji)
          ELSE
             ! There is not enough snow on the ice fraction to sustain the full sublimation demand.
             ! We take all the nobio snow and set the remaining sublimation demand on the vegetation fraction.
             ! We do this because the nobio snow cannot deal with a too high sublimation demand, whereas the 
             ! vegetation fraction can, through the subsinksoil variable (see below 2.8.1), which is taken into 
             ! account in the water budget (see TWBR in hydrol.f90).
             subsnownobio(ji,iice) = snow_nobio(ji,iice)
          ENDIF
          subsnowveg(ji) = vevapsno(ji) - subsnownobio(ji,iice)
       ELSE
          ! There is a small amount of snow.
          IF ( frac_nobio(ji,iice) .GT. min_sechiba) THEN
             ! The ice fraction is not too small. In this case, we make the hypothesis that the snow is mainly on the ice fraction.
             IF (snow_nobio(ji,iice) .GE. vevapsno(ji)) THEN
                ! There is enough snow on the ice fraction to sustain the sublimation demand.
                subsnownobio(ji,iice) = vevapsno(ji)
                subsnowveg(ji) = zero
             ELSE
                ! There is not enough snow on the ice fraction to sustain the sublimation demand.
                ! We take all the nobio snow and set the remaining sublimation demand on the vegetation fraction.
                ! We do this beacuse the nobio snow cannot deal with a too high sublimation demand, whereas the 
                ! vegetation fraction can, through the subsinksoil variable (see below 2.8.1), which is taken into 
                ! account in the water budget (see TWBR in hydrol.f90).
                subsnownobio(ji,iice) = snow_nobio(ji,iice)
                subsnowveg(ji) = vevapsno(ji) - subsnownobio(ji,iice)
             ENDIF
          ELSE
             ! The nobio snow fraction is too small, the vegetation fraction deals with the whole sublimation demand.
             subsnownobio(ji,iice) = zero
             subsnowveg(ji) = vevapsno(ji)
          ENDIF
       ENDIF

       !yx:+++++
       IF ( SUM(vevapsno_pft(ji,:)) .NE. 0) THEN
          DO jv=1,nvm
             !subsnowveg_pft(ji,jv) = (vevapsno_pft(ji,jv)/SUM(vevapsno_pft(ji,:)))*subsnowveg(ji) 
             !subsnowveg_pft(ji,jv) = subsnowveg(ji) ! for test, 2023/03/08
             !new
             !subsnowveg_pft(ji,jv) = (vevapsno(ji)-SUM(subsnownobio(ji,:)))/vevapsno(ji)*vevapsno_pft(ji,jv)
             !new3
             subsnowveg_pft(ji,jv) = (vevapsno_pft(ji,jv)/vevapsno(ji))*subsnowveg(ji)
          ENDDO
       ELSE
          subsnowveg_pft(ji,:) = zero
       ENDIF
       !yx:-----
       
       !! 2.8.1 Check that sublimation on the vegetated fraction is possible.
       IF (subsnowveg(ji) .GT. snow(ji)) THEN
          IF( (un - totfrac_nobio(ji)).GT.min_sechiba) THEN
             subsinksoil (ji) = (subsnowveg(ji) - snow(ji))/ (un - totfrac_nobio(ji))
          END IF
          ! Sublimation is thus limited to what is available
          subsnowveg(ji) = snow(ji)
          snow(ji) = zero
          snowdz(ji,:)  =  0
          snowliq(ji,:)   =  0
          snowtemp(ji,:) = tp_00 
          vevapsno(ji) = subsnowveg(ji) + subsnownobio(ji,iice)
       ELSE
          ! Calculating the snow accumulation  
          WSNOWDZ(ji,:)= snowdz(ji,:)*snowrho(ji,:)
          SMASSC (ji)= 0.0
          DO jj=1,nsnow
             SMASS(ji,jj)   = SMASSC(ji) + WSNOWDZ(ji,jj)
             SMASSC(ji)      = SMASSC(ji) + WSNOWDZ(ji,jj)
          ENDDO
          ! Finding the layer
          locjj=1
          DO jj=1,nsnow-1
             IF ((SMASS(ji,jj) .LE. subsnowveg(ji)) .AND. (SMASS(ji,jj+1) .GE. subsnowveg(ji)) ) THEN
                locjj=jj+1
             ENDIF
          ENDDO
          
          ! Calculating the removal of snow depth
          IF (locjj .EQ. 1) THEN
             ZSNOWEVAPS(ji)  = subsnowveg(ji)/snowrho(ji,1)
             ZSNOWDZ(ji)     = snowdz(ji,1) - ZSNOWEVAPS(ji)
             snowdz(ji,1)     = MAX(0.0, ZSNOWDZ(ji))
             IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) ) THEN
                WRITE(numout,*) 'yi debug: locjj=', locjj
                WRITE(numout,*) 'yi debug: ZSNOWDZ(ji)=', ZSNOWDZ(ji)
                WRITE(numout,*) 'yi debug: ZSNOWEVAPS(ji)=', ZSNOWEVAPS(ji)
                WRITE(numout,*) 'yi debug: subsnowveg(ji)=', subsnowveg(ji)
                WRITE(numout,*) 'yi debug: snowrho(ji,1)=', snowrho(ji,1)
                WRITE(numout,*) 'yi debug: snowdz(ji,1)=', snowdz(ji,1)
             ENDIF
          ELSE IF (locjj .GT. 1) THEN
             snowacc(ji)=0
             DO jj=1,locjj-1
                snowacc(ji)=snowacc(ji)+snowdz(ji,jj)*snowrho(ji,jj)
                snowdz(ji,jj)=0
             ENDDO
             ZSNOWEVAPS(ji)  = (subsnowveg(ji)-snowacc(ji))/snowrho(ji,locjj)
             ZSNOWDZ(ji)     = snowdz(ji,locjj) - ZSNOWEVAPS(ji)
             snowdz(ji,locjj)     = MAX(0.0, ZSNOWDZ(ji))
             IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) ) THEN
                WRITE(numout,*) 'yi debug: locjj=', locjj
                WRITE(numout,*) 'yi debug: ZSNOWDZ(ji,2)=', ZSNOWDZ(ji)
                WRITE(numout,*) 'yi debug: ZSNOWEVAPS(ji)=', ZSNOWEVAPS(ji)
                WRITE(numout,*) 'yi debug: subsnowveg(ji)=', subsnowveg(ji)
                WRITE(numout,*) 'yi debug: snowacc(ji)=', snowacc(ji)
                WRITE(numout,*) 'yi debug: snowrho(ji,locjj)=', snowrho(ji,locjj)
                WRITE(numout,*) 'yi debug: snowdz(ji,locjj)=', snowdz(ji,locjj)
             ENDIF
!          ELSE
!             ZSNOWEVAPS(ji)  = subsnowveg(ji)/snowrho(ji,1)
!             ZSNOWDZ(ji)     = snowdz(ji,1) - ZSNOWEVAPS(ji)
!             snowdz(ji,1)     = MAX(0.0, ZSNOWDZ(ji))
          ENDIF
          
       ENDIF

       !yx:+++++
       DO jv=1,nvm
          IF (subsnowveg_pft(ji,jv) .GT. snow_pft(ji,jv)) THEN
             IF( (un - totfrac_nobio(ji)).GT.min_sechiba) THEN
                subsinksoil_pft (ji,jv) = (subsnowveg_pft(ji,jv) - snow_pft(ji,jv)) !/ (un - totfrac_nobio(ji)) !yx:+- commented
             END IF
             ! Sublimation is thus limited to what is available
             subsnowveg_pft(ji,jv) = snow_pft(ji,jv)
             snow_pft(ji,jv) = zero
             snowdz_pft(ji,:,jv)  =  0
             snowliq_pft(ji,:,jv)   =  0
             snowtemp_pft(ji,:,jv) = tp_00
             vevapsno_pft(ji,jv) = subsnowveg_pft(ji,jv)
          ELSE
             ! Calculating the snow accumulation
             WSNOWDZ_pft(ji,:,jv)= snowdz_pft(ji,:,jv)*snowrho_pft(ji,:,jv)
             SMASSC_pft(ji,jv)= 0.0
             DO jj=1,nsnow
                SMASS_pft(ji,jj,jv)   = SMASSC_pft(ji,jv) + WSNOWDZ_pft(ji,jj,jv)
                SMASSC_pft(ji,jv)      = SMASSC_pft(ji,jv) + WSNOWDZ_pft(ji,jj,jv)
             ENDDO
             ! Finding the layer
             locjj=1
             DO jj=1,nsnow-1
                IF ((SMASS_pft(ji,jj,jv) .LE. subsnowveg_pft(ji,jv)) .AND. (SMASS_pft(ji,jj+1,jv) .GE. subsnowveg_pft(ji,jv)) ) THEN
                   locjj=jj+1
                ENDIF
             ENDDO

             ! Calculating the removal of snow depth
             IF (locjj .EQ. 1) THEN
                ZSNOWEVAPS_pft(ji,jv)  = subsnowveg_pft(ji,jv)/snowrho_pft(ji,1,jv)
                ZSNOWDZ_pft(ji,jv)     = snowdz_pft(ji,1,jv) - ZSNOWEVAPS_pft(ji,jv)
                snowdz_pft(ji,1,jv)     = MAX(0.0, ZSNOWDZ_pft(ji,jv))
                !IF ( isnan(snowdz_pft(ji,1,jv)) ) THEN
                IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) .AND. (jv .EQ. 2) ) THEN
                   WRITE(numout,*) 'yi debug: locjj=', locjj
                   WRITE(numout,*) 'yi debug: ZSNOWDZ_pft(ji,2)=', ZSNOWDZ_pft(ji,jv)
                   WRITE(numout,*) 'yi debug: ZSNOWEVAPS_pft(ji,2)=', ZSNOWEVAPS_pft(ji,jv)
                   WRITE(numout,*) 'yi debug: subsnowveg_pft(ji,2)=', subsnowveg_pft(ji,jv)
                   WRITE(numout,*) 'yi debug: snowrho_pft(ji,1,2)=', snowrho_pft(ji,1,jv)
                   WRITE(numout,*) 'yi debug: snowdz_pft(ji,1,2)=', snowdz_pft(ji,1,jv)
                ENDIF
             ELSE IF (locjj .GT. 1) THEN
                snowacc_pft(ji,jv)=0
                DO jj=1,locjj-1
                   snowacc_pft(ji,jv)=snowacc_pft(ji,jv)+snowdz_pft(ji,jj,jv)*snowrho_pft(ji,jj,jv)
                   snowdz_pft(ji,jj,jv)=0
                ENDDO
                ZSNOWEVAPS_pft(ji,jv)  = (subsnowveg_pft(ji,jv)-snowacc_pft(ji,jv))/snowrho_pft(ji,locjj,jv)
                ZSNOWDZ_pft(ji,jv)     = snowdz_pft(ji,locjj,jv) - ZSNOWEVAPS_pft(ji,jv)
                snowdz_pft(ji,locjj,jv)     = MAX(0.0, ZSNOWDZ_pft(ji,jv))
                IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) .AND. (jv .EQ. 2) ) THEN
                   WRITE(numout,*) 'yi debug: locjj=', locjj
                   WRITE(numout,*) 'yi debug: ZSNOWDZ_pft(ji,2)=', ZSNOWDZ_pft(ji,jv)
                   WRITE(numout,*) 'yi debug: ZSNOWEVAPS_pft(ji,2)=', ZSNOWEVAPS_pft(ji,jv)
                   WRITE(numout,*) 'yi debug: subsnowveg_pft(ji,2)=', subsnowveg_pft(ji,jv)
                   WRITE(numout,*) 'yi debug: snowacc_pft(ji,2)=', snowacc_pft(ji,jv)
                   WRITE(numout,*) 'yi debug: snowrho_pft(ji,locjj,2)=', snowrho_pft(ji,locjj,jv)
                   WRITE(numout,*) 'yi debug: snowdz_pft(ji,locjj,2)=', snowdz_pft(ji,locjj,jv)
                ENDIF
   !          ELSE
   !             ZSNOWEVAPS(ji)  = subsnowveg(ji)/snowrho(ji,1)
   !             ZSNOWDZ(ji)     = snowdz(ji,1) - ZSNOWEVAPS(ji)
   !             snowdz(ji,1)     = MAX(0.0, ZSNOWDZ(ji))
             ENDIF

          ENDIF
       ENDDO
       !yx:-----

    ENDDO

    IF (writeforcheck_explicitsnow) THEN
       WRITE(numout,*) 'after 2.9 Snow sublimation changing snow thickness'
       WRITE(numout,*) 'yi debug: snowdz(1,:)=', snowdz(1,:)
       WRITE(numout,*) 'yi debug: snowdz_pft(1,:,:)=', snowdz_pft(1,:,:)
    ENDIF

    
! 2.9.1 Snowmelt_from_maxmass 

     DO ji=1,kjpindex !domain size
       snow(ji) = SUM(snowrho(ji,:) * snowdz(ji,:))
       IF(snow(ji).GT.maxmass_snow)THEN
           IF (snow(ji).GT.(1.2*maxmass_snow)) THEN 
              ! melt faster for points where accumulation is too high
              snow_d1k = trois * soilcap(ji) / chalfu0
           ELSE
              ! standard melting
              snow_d1k = un * soilcap(ji) / chalfu0
           ENDIF
           snowmelt_from_maxmass(ji) = MIN((snow(ji) - maxmass_snow),snow_d1k)
           ! Calculating the snow accumulation  
           WSNOWDZ(ji,:)= snowdz(ji,:)*snowrho(ji,:) ! WSNOWDZ en kg/m2
           SMASSC (ji)= 0.0
           DO jj=nsnow,1,-1
             SMASS(ji,jj)   = SMASSC(ji) + WSNOWDZ(ji,jj)
             SMASSC(ji)      = SMASSC(ji) + WSNOWDZ(ji,jj)
           ENDDO

          ! Finding the layer
          locjj = nsnow
          DO jj=nsnow,2,-1
             IF ((SMASS(ji,jj) .LE. snowmelt_from_maxmass(ji)) .AND. (SMASS(ji,jj-1) .GE. snowmelt_from_maxmass(ji)) ) THEN
                locjj=jj-1
             ENDIF
          ENDDO
          
          ! Calculating the removal of snow depth
          IF (locjj .EQ. 3) THEN
! ZSNOWMELT_XS : Epaisseur de la couche qui a disparu partiellement 
             ZSNOWMELT_XS(ji)  = snowmelt_from_maxmass(ji)/snowrho(ji,3)
             ZSNOWDZ(ji)     = snowdz(ji,3) - ZSNOWMELT_XS(ji)
             snowdz(ji,3)     = MAX(0.0, ZSNOWDZ(ji))
          ELSE IF (locjj .LT. 3) THEN
             snow_remove(ji)=0   ! masse de neige des couches qui disparaissent totalement
             DO jj=3,locjj+1,-1
                snow_remove(ji)=snow_remove(ji)+snowdz(ji,jj)*snowrho(ji,jj)
                snowdz(ji,jj)=0
             ENDDO
             ZSNOWMELT_XS(ji)  = (snowmelt_from_maxmass(ji) - snow_remove(ji))/snowrho(ji,locjj)
             ZSNOWDZ(ji)     = snowdz(ji,locjj) - ZSNOWMELT_XS(ji)
             snowdz(ji,locjj)     = MAX(0.0, ZSNOWDZ(ji))             
          ENDIF
       ENDIF
     
       !yx:+++++
       DO jv=1,nvm
          snow_pft(ji,jv) = SUM(snowrho_pft(ji,:,jv) * snowdz_pft(ji,:,jv))
          IF(snow_pft(ji,jv).GT.maxmass_snow)THEN
             IF (snow_pft(ji,jv).GT.(1.2*maxmass_snow)) THEN
                ! melt faster for points where accumulation is too high
                snow_d1k = trois * soilcap_pft(ji,jv) / chalfu0
             ELSE
                ! standard melting
                snow_d1k = un * soilcap_pft(ji,jv) / chalfu0
             ENDIF
             snowmelt_from_maxmass_pft(ji,jv) = MIN((snow_pft(ji,jv) - maxmass_snow),snow_d1k)
             ! Calculating the snow accumulation
             WSNOWDZ_pft(ji,:,jv)= snowdz_pft(ji,:,jv)*snowrho_pft(ji,:,jv) ! WSNOWDZ en kg/m2
             SMASSC_pft(ji,jv)= 0.0
             DO jj=nsnow,1,-1
                SMASS_pft(ji,jj,jv)   = SMASSC_pft(ji,jv) + WSNOWDZ_pft(ji,jj,jv)
                SMASSC_pft(ji,jv)      = SMASSC_pft(ji,jv) + WSNOWDZ_pft(ji,jj,jv)
             ENDDO

             ! Finding the layer
             locjj = nsnow
             DO jj=nsnow,2,-1
                IF ((SMASS_pft(ji,jj,jv) .LE. snowmelt_from_maxmass_pft(ji,jv)) .AND. (SMASS_pft(ji,jj-1,jv) .GE. snowmelt_from_maxmass_pft(ji,jv)) ) THEN
                   locjj=jj-1
                ENDIF
             ENDDO

             ! Calculating the removal of snow depth
             IF (locjj .EQ. 3) THEN
! ZSNOWMELT_XS : Epaisseur de la couche qui a disparu partiellement
                 ZSNOWMELT_XS_pft(ji,jv)  = snowmelt_from_maxmass_pft(ji,jv)/snowrho_pft(ji,3,jv)
                 ZSNOWDZ_pft(ji,jv)     = snowdz_pft(ji,3,jv) - ZSNOWMELT_XS_pft(ji,jv)
                 snowdz_pft(ji,3,jv)     = MAX(0.0, ZSNOWDZ_pft(ji,jv))
             ELSE IF (locjj .LT. 3) THEN
                snow_remove_pft(ji,jv)=0   ! masse de neige des couches qui disparaissent totalement
                DO jj=3,locjj+1,-1
                   snow_remove_pft(ji,jv)=snow_remove_pft(ji,jv)+snowdz_pft(ji,jj,jv)*snowrho_pft(ji,jj,jv)
                   snowdz_pft(ji,jj,jv)=0
                ENDDO
                ZSNOWMELT_XS_pft(ji,jv)  = (snowmelt_from_maxmass_pft(ji,jv) - snow_remove_pft(ji,jv))/snowrho_pft(ji,locjj,jv)
                ZSNOWDZ_pft(ji,jv)     = snowdz_pft(ji,locjj,jv) - ZSNOWMELT_XS_pft(ji,jv)
                snowdz_pft(ji,locjj,jv)     = MAX(0.0, ZSNOWDZ_pft(ji,jv))
             ENDIF
          ENDIF
       ENDDO
       !yx:-----
 
    ENDDO   

    IF (writeforcheck_explicitsnow) THEN
       WRITE(numout,*) 'after 2.9.1 Snowmelt_from_maxmass'
       WRITE(numout,*) 'yi debug: snowdz(1,:)=', snowdz(1,:)
       WRITE(numout,*) 'yi debug: snowdz_pft(1,:,:)=', snowdz_pft(1,:,:)
    ENDIF
    
    !2.10 Calculate snow grain size using the updated thermal gradient
    
    CALL explicitsnow_grain(kjpindex,snowliq,snowdz,gtemp,snowtemp,pb,snowgrain,&
         snowliq_pft, snowdz_pft, gtemp_pft, snowtemp_pft, snowgrain_pft) !yx:+-
    
    IF (writeforcheck_explicitsnow) THEN
       WRITE(numout,*) 'after explicitsnow_grain'
       WRITE(numout,*) 'yi debug: snowgrain(1,:)=', snowgrain(1,:)
       WRITE(numout,*) 'yi debug: snowgrain_pft(1,:,:)=', snowgrain_pft(1,:,:)
    ENDIF

    !2.11  Update snow heat
    ! Update the heat content (variable stored each time step)
    ! using current snow temperature and liquid water content:
    !
    ! First, make check to make sure heat content not too large
    ! (this can result due to signifigant heating of thin snowpacks):
    ! add any excess heat to ground flux:
    !
    DO ji=1,kjpindex
       DO jj=1,nsnow
          ZLIQHEATXS(ji)  = MAX(0.0, snowliq(ji,jj)*ph2o - 0.10*snowdz(ji,jj)*snowrho(ji,jj))*chalfu0/dt_sechiba
          snowliq(ji,jj) = snowliq(ji,jj) - ZLIQHEATXS(ji)*dt_sechiba/(ph2o*chalfu0)
          snowliq(ji,jj) = MAX(0.0, snowliq(ji,jj))
          grndflux(ji)   = grndflux(ji)   + ZLIQHEATXS(ji)
       ENDDO

       !yx:+++++
       DO jv=1,nvm
          DO jj=1,nsnow
             ZLIQHEATXS_pft(ji,jv)  = MAX(0.0, snowliq_pft(ji,jj,jv)*ph2o - 0.10*snowdz_pft(ji,jj,jv)*snowrho_pft(ji,jj,jv))*chalfu0/dt_sechiba
             snowliq_pft(ji,jj,jv) = snowliq_pft(ji,jj,jv) - ZLIQHEATXS_pft(ji,jv)*dt_sechiba/(ph2o*chalfu0)
             snowliq_pft(ji,jj,jv) = MAX(0.0, snowliq_pft(ji,jj,jv))
             grndflux_pft(ji,jv)   = grndflux_pft(ji,jv)   + ZLIQHEATXS_pft(ji,jv)
          ENDDO
       ENDDO
       !yx:-----
    ENDDO
    
    snow(:) = 0.0
    snow_pft(:, :) = 0.0 !yx:+-
    DO ji=1,kjpindex !domain size
       snow(ji) = SUM(snowrho(ji,:) * snowdz(ji,:))
       !yx:+++++
       DO jv=1,nvm
          snow_pft(ji,jv) = SUM(snowrho_pft(ji,:,jv) * snowdz_pft(ji,:,jv))
       ENDDO
       !yx:-----
    ENDDO
        
    DO ji = 1, kjpindex
       snowheat(ji,:) = snow3lheat_1d(snowliq(ji,:),snowrho(ji,:),snowdz(ji,:),snowtemp(ji,:))
       !yx:+++++
       DO jv=1,nvm
          snowheat_pft(ji,:,jv) = snow3lheat_1d(snowliq_pft(ji,:,jv),snowrho_pft(ji,:,jv),snowdz_pft(ji,:,jv),snowtemp_pft(ji,:,jv)) !yx:+-
       ENDDO
       !yx:-----
    ENDDO
    
    !3. on land ice (using the default ORCHIDEE snow scheme)
    
    DO ji = 1,kjpindex
       
       !! 3.1. It is snowing
       snow_nobio(ji,iice) = snow_nobio(ji,iice) + frac_nobio(ji,iice)*precip_snow(ji) + &
            & frac_nobio(ji,iice)*precip_rain(ji)
       
       !! 3.2. Sublimation - was calculated before, it cannot give us negative snow_nobio (see 2.9).
       snow_nobio(ji,iice) = snow_nobio(ji,iice) - subsnownobio(ji,iice)
       
       !! 3.3. ice melt only for continental ice fraction
       snowmelt_tmp(ji) = zero
       IF (temp_sol_new_old(ji) .GT. tp_00) THEN
          
          !! 3.3.1 If there is snow on the ice-fraction it can melt
          
          snowmelt_tmp(ji) = frac_nobio(ji,iice)*(temp_sol_new_old(ji) - tp_00) * soilcap(ji) / chalfu0
          
          IF ( snowmelt_tmp(ji) .GT. snow_nobio(ji,iice) ) THEN
             snowmelt_tmp(ji) = MAX( 0., snow_nobio(ji,iice))
          ENDIF
          snowmelt_ice(ji) = snowmelt_ice(ji) + snowmelt_tmp(ji)
          snow_nobio(ji,iice) = snow_nobio(ji,iice) - snowmelt_tmp(ji)
          
       ENDIF


       !! Ice melt only if there is more than a given mass : maxmass_snow, i.e. only weight melts glaciers !
       
       IF ( snow_nobio(ji,iice) .GE. maxmass_snow ) THEN
          icemelt(ji) = snow_nobio(ji,iice) - maxmass_snow
          snow_nobio(ji,iice) = maxmass_snow
       ENDIF
       
    END DO
    
    
    !! 4. On other surface types - not done yet
    
    IF ( nnobio .GT. 1 ) THEN
       WRITE(numout,*) 'WE HAVE',nnobio-1,' SURFACE TYPES I DO NOT KNOW'
       WRITE(numout,*) 'CANNOT TREAT SNOW ON THESE SURFACE TYPES'
       CALL ipslerr_p(3,'explicitsnow_main','Cannot treat snow on these surface types.','','')
    ENDIF
    
    !! 5. Computes snow age on land and land ice (for albedo)
    
    DO ji = 1, kjpindex
       
       !! 5.1. Snow age on land 
       
       IF (snow(ji) .LE. zero) THEN
          snow_age(ji) = zero
       ELSE
          snow_age(ji) =(snow_age(ji) + (un - snow_age(ji)/max_snow_age) * dt_sechiba/one_day) &
               & * EXP(-precip_snow(ji) / snow_trans)
       ENDIF
       !yx:+++++
       DO jv=1,nvm
          IF (snow_pft(ji,jv) .LE. zero) THEN
             snow_age_pft(ji,jv) = zero
          ELSE
             snow_age_pft(ji,jv) =(snow_age_pft(ji,jv) + (un - snow_age_pft(ji,jv)/max_snow_age) * dt_sechiba/one_day) &
                          & * EXP(-precip_snow(ji) / snow_trans)
          ENDIF
       ENDDO
       !yx:-----
       
       !! 5.2. Snow age on land ice
       
       !! Age of snow on ice: a little bit different because in cold regions, we really
       !! cannot negect the effect of cold temperatures on snow metamorphism any more.
       
       IF (snow_nobio(ji,iice) .LE. zero) THEN
          snow_nobio_age(ji,iice) = zero
       ELSE
          
          d_age(ji) = ( snow_nobio_age(ji,iice) + &
               &  (un - snow_nobio_age(ji,iice)/max_snow_age) * dt_sechiba/one_day ) * &
               &  EXP(-precip_snow(ji) / snow_trans) - snow_nobio_age(ji,iice)
          IF (d_age(ji) .GT. 0. ) THEN
             xx(ji) = MAX( tp_00 - temp_sol_new(ji), zero )
             xx(ji) = ( xx(ji) / 7._r_std ) ** 4._r_std
             d_age(ji) = d_age(ji) / (un+xx(ji))
          ENDIF
          snow_nobio_age(ji,iice) = MAX( snow_nobio_age(ji,iice) + d_age(ji), zero )
          
       ENDIF

    ENDDO

    
    
    !! 6. Check the snow on land 
    DO ji=1,kjpindex
       IF (snow(ji) .EQ. 0) THEN
          snowrho(ji,:)=50.0
          snowgrain(ji,:)=0.0
          snowdz(ji,:)=0.0
          snowliq(ji,:)=0.0
       ENDIF
       !yx:+++++
       DO jv=1,nvm
          IF (snow_pft(ji,jv) .EQ. 0) THEN
             snowrho_pft(ji,:,jv)=50.0
             snowgrain_pft(ji,:,jv)=0.0
             snowdz_pft(ji,:,jv)=0.0
             snowliq_pft(ji,:,jv)=0.0
          ENDIF
       ENDDO
       !yx:-----
    ENDDO
    
    
    ! Snow melt only if there is more than a given mass : maxmass_snow
    ! Here I suggest to remove the snow based on a certain threshold of snow
    ! depth instead of snow mass because it is quite difficult for
    ! explictsnow module to calculate other snow properties following the
    ! removal of snow mass
    ! to define the threshold of snow depth based on old snow density (330
    ! kg/m3)
    !       maxmass_snowdepth=maxmass_snow/sn_dens 
!    snowmelt_from_maxmass(:) = 0.0
    
    !! 7. compute total melt 
    
    DO ji=1,kjpindex 
       tot_melt(ji) = icemelt(ji) + snowmelt(ji) + snowmelt_ice(ji) + snowmelt_from_maxmass(ji)
       tot_melt_pft(ji,:) = snowmelt_pft(ji,:) + snowmelt_from_maxmass_pft(ji,:) !yx:+-
       !yx:+- here for each pft, I don't consider ice melt, ice is for nobio.
       !tot_melt is used in enerbil_fusion and when ok_explicitsnow = .FALSE. ,
       !so it won't affect my simulation. 2023/02/28
    ENDDO
    IF (printlev>=3) WRITE(numout,*) 'explicitsnow_main done'
    
    ! Write output variables
    CALL xios_orchidee_send_field("snowliq",snowliq)
    CALL xios_orchidee_send_field("snowrho",snowrho)
    CALL xios_orchidee_send_field("snowheat",snowheat)
    CALL xios_orchidee_send_field("snowgrain",snowgrain)
    CALL xios_orchidee_send_field("snowmelt_from_maxmass",snowmelt_from_maxmass)
    CALL xios_orchidee_send_field("soilcap",soilcap)

    !yx:+++++
    DO jv = 1,nvm
       IF (.NOT. (ok_LAIdev_sechiba(jv))) THEN
          snowmelt_pft(:,jv) = snowmelt(:)
          tot_melt_pft(:,jv) = tot_melt(:)
          vevapsno_pft(:,jv) = vevapsno(:)
          subsinksoil_pft(:,jv) = subsinksoil(:)
          snow_age_pft(:,jv) = snow_age(:)
          snowtemp_pft(:,:,jv) = snowtemp(:,:)
          snowrho_pft(:,:,jv) = snowrho(:,:)
          snowgrain_pft(:,:,jv) = snowgrain(:,:)
          snowdz_pft(:,:,jv) = snowdz(:,:)
          snowheat_pft(:,:,jv) = snowheat(:,:)
          snowliq_pft(:,:,jv) = snowliq(:,:)
          snow_pft(:,jv) = snow(:)
          grndflux_pft(:,jv) = grndflux(:)
          temp_sol_add_pft(:,jv) = temp_sol_add(:)
       ENDIF
    ENDDO
    IF (ANY(ok_LAIdev_sechiba)) THEN
       snowmelt(:) = SUM(snowmelt_pft(:,:)*veget_max(:,:),DIM=2)/SUM(veget_max(:,:),DIM=2)
       tot_melt(:) = SUM(tot_melt_pft(:,:)*veget_max(:,:),DIM=2)/SUM(veget_max(:,:),DIM=2)
       vevapsno(:) = SUM(vevapsno_pft(:,:)*veget_max(:,:),DIM=2)/SUM(veget_max(:,:),DIM=2)
       subsinksoil(:) = SUM(subsinksoil_pft(:,:)*veget_max(:,:),DIM=2)/SUM(veget_max(:,:),DIM=2)
       snow_age(:) = SUM(snow_age_pft(:,:)*veget_max(:,:),DIM=2)/SUM(veget_max(:,:),DIM=2)

       DO jj = 1,nsnow
          snowtemp(:,jj) = zero
          snowrho(:,jj) = zero
          snowgrain(:,jj) = zero
          snowdz(:,jj) = zero
          snowheat(:,jj) = zero
          snowliq(:,jj) = zero
          DO jv = 1,nvm
             snowtemp(:,jj) = snowtemp(:,jj) + snowtemp_pft(:,jj,jv)*veget_max(:,jv)/SUM(veget_max(:,:),DIM=2)
             snowrho(:,jj) = snowrho(:,jj) + snowrho_pft(:,jj,jv)*veget_max(:,jv)/SUM(veget_max(:,:),DIM=2)
             snowgrain(:,jj) = snowgrain(:,jj) + snowgrain_pft(:,jj,jv)*veget_max(:,jv)/SUM(veget_max(:,:),DIM=2)
             snowdz(:,jj) = snowdz(:,jj) + snowdz_pft(:,jj,jv)*veget_max(:,jv)/SUM(veget_max(:,:),DIM=2)
             snowheat(:,jj) = snowheat(:,jj) + snowheat_pft(:,jj,jv)*veget_max(:,jv)/SUM(veget_max(:,:),DIM=2)
             snowliq(:,jj) = snowliq(:,jj) + snowliq_pft(:,jj,jv)*veget_max(:,jv)/SUM(veget_max(:,:),DIM=2)
          ENDDO
       ENDDO

       snow(:) = SUM(snow_pft(:,:)*veget_max(:,:),DIM=2)/SUM(veget_max(:,:),DIM=2)
       grndflux(:) = SUM(grndflux_pft(:,:)*veget_max(:,:),DIM=2)/SUM(veget_max(:,:),DIM=2)
       temp_sol_add(:) = SUM(temp_sol_add_pft(:,:)*veget_max(:,:),DIM=2)/SUM(veget_max(:,:),DIM=2)
    ENDIF
    !yx:-----

  END SUBROUTINE explicitsnow_main
  

!================================================================================================================================
!! SUBROUTINE   : explicitsnow_finalize
!!
!>\BRIEF        Write variables for explictsnow module to restart file
!!                
!! DESCRIPTION  : Write variables for explictsnow module to restart file
!!
!! \n
!_
!================================================================================================================================
  SUBROUTINE explicitsnow_finalize ( kjit,     kjpindex, rest_id,    snowrho,   &
       snowtemp, snowdz,   snowheat,   snowgrain, &
       snowrho_pft, snowtemp_pft, snowdz_pft, snowheat_pft, snowgrain_pft) !yx:+-
    
    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                           :: kjit           !! Time step number 
    INTEGER(i_std), INTENT(in)                           :: kjpindex       !! Domain size
    INTEGER(i_std),INTENT (in)                           :: rest_id        !! Restart file identifier
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(in)  :: snowrho        !! Snow density (Kg/m^3)
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(in)  :: snowtemp       !! Snow temperature
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(in)  :: snowdz         !! Snow layer thickness
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(in)  :: snowheat       !! Snow heat content (J/m^2)
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(in)  :: snowgrain      !! Snow grainsize (m)
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(in)  :: snowrho_pft        !! Snow density (Kg/m^3) !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(in)  :: snowtemp_pft       !! Snow temperature !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(in)  :: snowdz_pft         !! Snow layer thickness !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(in)  :: snowheat_pft       !! Snow heat content (J/m^2) !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(in)  :: snowgrain_pft      !! Snow grainsize (m) !yx:+-
    
    !! 1. Write to restart file
    CALL restput_p(rest_id, 'snowrho', nbp_glo, nsnow, 1, kjit, snowrho, 'scatter', nbp_glo, index_g)
    CALL restput_p(rest_id, 'snowtemp', nbp_glo, nsnow, 1, kjit, snowtemp, 'scatter', nbp_glo, index_g)
    CALL restput_p(rest_id, 'snowdz', nbp_glo, nsnow, 1, kjit, snowdz, 'scatter', nbp_glo, index_g)
    CALL restput_p(rest_id, 'snowheat', nbp_glo, nsnow, 1, kjit, snowheat, 'scatter', nbp_glo, index_g)
    CALL restput_p(rest_id, 'snowgrain', nbp_glo, nsnow, 1, kjit, snowgrain, 'scatter', nbp_glo, index_g)
    CALL restput_p(rest_id, 'snowrho_pft', nbp_glo, nsnow, nvm, kjit, snowrho_pft, 'scatter', nbp_glo, index_g) !yx:+-
    CALL restput_p(rest_id, 'snowtemp_pft', nbp_glo, nsnow, nvm, kjit, snowtemp_pft, 'scatter', nbp_glo, index_g) !yx:+-
    CALL restput_p(rest_id, 'snowdz_pft', nbp_glo, nsnow, nvm, kjit, snowdz_pft, 'scatter', nbp_glo, index_g) !yx:+-
    CALL restput_p(rest_id, 'snowheat_pft', nbp_glo, nsnow, nvm, kjit, snowheat_pft, 'scatter', nbp_glo, index_g) !yx:+-
    CALL restput_p(rest_id, 'snowgrain_pft', nbp_glo, nsnow, nvm, kjit, snowgrain_pft, 'scatter', nbp_glo, index_g) !yx:+-
    
  END SUBROUTINE explicitsnow_finalize
  

!================================================================================================================================
!! SUBROUTINE   : explicitsnow_grain
!!
!>\BRIEF        Compute evolution of snow grain size
!!                
!! DESCRIPTION  : 
!!
!! RECENT CHANGE(S) : None 
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : R. Jordan (1991)
!!
!! FLOWCHART    : None
!! \n
!_
!================================================================================================================================


 SUBROUTINE explicitsnow_grain(kjpindex,snowliq,snowdz,gtemp,snowtemp,pb,snowgrain,&
            snowliq_pft, snowdz_pft, gtemp_pft, snowtemp_pft, snowgrain_pft) !yx:+-

      !! 0.1 Input variables
      INTEGER(i_std),INTENT(in)                                  :: kjpindex         !! Domain size
      REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(in)           :: snowliq          !! Liquid water content
      REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(in)       :: snowliq_pft      !! Liquid water content !yx:+-
      REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(in)           :: snowdz           !! Snow depth (m)
      REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(in)       :: snowdz_pft       !! Snow depth (m) !yx:+-
      REAL(r_std),DIMENSION(kjpindex),INTENT(in)                 :: gtemp            !! First soil layer temperature
      REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)             :: gtemp_pft        !! First soil layer temperature !yx:+-
      REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(in)           :: snowtemp         !! Snow temperature
      REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(in)       :: snowtemp_pft     !! Snow temperature !yx:+-
      REAL(r_std),DIMENSION (kjpindex),INTENT(in)                :: pb               !! Surface pressure (hpa) 

      !! 0.2 Output variables

      !! 0.3 Modified variables

      REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(inout)        :: snowgrain        !! Snow grain size (m)
      REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(inout)    :: snowgrain_pft    !! Snow grain size (m) !yx:+-

      !! 0.4 Local variables
      REAL(r_std),DIMENSION(kjpindex,nsnow)                      :: zsnowdz,zdz,ztheta
      REAL(r_std),DIMENSION(kjpindex,nsnow,nvm)                  :: zsnowdz_pft,zdz_pft,ztheta_pft !yx:+-
      REAL(r_std),DIMENSION(kjpindex,0:nsnow)                    :: ztemp,zdiff,ztgrad,zthetaa,zfrac,&
                                                                    zexpo,zckt_liq,zckt_ice,zckt
      !yx:+++++
      REAL(r_std),DIMENSION(kjpindex,0:nsnow,nvm)                :: ztemp_pft,zdiff_pft,ztgrad_pft,zthetaa_pft,zfrac_pft, &
                                                                    zexpo_pft,zckt_liq_pft,zckt_ice_pft,zckt_pft
      !yx:-----
      REAL(r_std),DIMENSION(kjpindex,nsnow)                      :: zrhomin,zgrainmin
      REAL(r_std),DIMENSION(kjpindex,nsnow,nvm)                  :: zrhomin_pft,zgrainmin_pft !yx:+-
      INTEGER(i_std) :: ji,jj,jv !yx:+- add jv

      !! 0.5 Local parameters
      REAL(r_std), PARAMETER                                     :: ztheta_crit = 0.02     !! m3 m-3
      REAL(r_std), PARAMETER                                     :: zc1_ice     = 8.047E+9 !! kg m-3 K
      REAL(r_std), PARAMETER                                     :: zc1_liq     = 5.726E+8 !! kg m-3 K
      REAL(r_std), PARAMETER                                     :: zdeos       = 0.92E-4  !! effective diffusion
                                                                                           !! coef for water vapor in snow
                                                                                           !! at 0C and 1000 mb (m2 s-1)
      REAL(r_std), PARAMETER                                     :: zg1         = 5.0E-7   !! m4 kg-1
      REAL(r_std), PARAMETER                                     :: zg2         = 4.0E-12  !! m2 s-1
      REAL(r_std), PARAMETER                                     :: ztheta_w      = 0.05   !! m3 m-3
      REAL(r_std), PARAMETER                                     :: ztheta_crit_w = 0.14   !! m3 m-3
      REAL(r_std), PARAMETER                                     :: zdzmin        = 0.01   !! m : minimum thickness
                                                                                           !! for thermal gradient evaluation:
                                                                                           !! to prevent excessive gradients
                                                                                           !! for vanishingly thin snowpacks.
      REAL(r_std), PARAMETER                                     :: xp00=1.E5 
      !! 1. initialize
 
      DO ji=1,kjpindex 


         zsnowdz(ji,:)  = MAX(xsnowdmin/nsnow, snowdz(ji,:))
         
         zsnowdz_pft(ji,:,:)  = MAX(xsnowdmin/nsnow, snowdz_pft(ji,:,:)) !yx:+-

         IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) ) THEN
            WRITE(numout,*) 'zsnowdz(1,:)=',zsnowdz(1,:)
            WRITE(numout,*) 'zsnowdz_pft(1,:,:)=',zsnowdz_pft(1,:,:)
         ENDIF

         DO jj=1,nsnow-1
            zdz(ji,jj)      = zsnowdz(ji,jj) + zsnowdz(ji,jj+1)
         ENDDO
            zdz(ji,nsnow)     = zsnowdz(ji,nsnow)

         !yx:+++++
         DO jv=1,nvm
            DO jj=1,nsnow-1
               zdz_pft(ji,jj,jv)      = zsnowdz_pft(ji,jj,jv) + zsnowdz_pft(ji,jj+1,jv)
            ENDDO
            zdz_pft(ji,nsnow,jv)     = zsnowdz_pft(ji,nsnow,jv)
         ENDDO
         !yx:-----

         IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) ) THEN
            WRITE(numout,*) 'zdz(1,:)=',zdz(1,:)
            WRITE(numout,*) 'zdz_pft(1,:,:)=',zdz_pft(1,:,:)
         ENDIF

         ! compute interface average volumetric water content (m3 m-3):
         ! first, layer avg VWC:
         !
          ztheta(ji,:) = snowliq(ji,:)/MAX(xsnowdmin, zsnowdz(ji,:))
          ztheta_pft(ji,:,:) = snowliq_pft(ji,:,:)/MAX(xsnowdmin, zsnowdz_pft(ji,:,:)) !yx:+-
          IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) ) THEN
             WRITE(numout,*) 'ztheta(1,:)=',ztheta(1,:)
             WRITE(numout,*) 'ztheta_pft(1,:,:)=',ztheta_pft(1,:,:)
          ENDIF

         ! at interfaces:
          zthetaa(ji,0)      = ztheta(ji,1)
          DO jj=1,nsnow-1
             zthetaa(ji,jj)  = (zsnowdz(ji,jj)  *ztheta(ji,jj)   +             &
                                   zsnowdz(ji,jj+1)*ztheta(ji,jj+1))/zdz(ji,jj)
          ENDDO
          zthetaa(ji,nsnow) = ztheta(ji,nsnow)

          !yx:+++++
          DO jv=1,nvm
             zthetaa_pft(ji,0,jv)      = ztheta_pft(ji,1,jv)
             DO jj=1,nsnow-1
                zthetaa_pft(ji,jj,jv)  = (zsnowdz_pft(ji,jj,jv)  *ztheta_pft(ji,jj,jv)   +             &
                                      zsnowdz_pft(ji,jj+1,jv)*ztheta_pft(ji,jj+1,jv))/zdz_pft(ji,jj,jv)
             ENDDO
             zthetaa_pft(ji,nsnow,jv) = ztheta_pft(ji,nsnow,jv)
          ENDDO
          !yx:-----
          IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) ) THEN
             WRITE(numout,*) 'zthetaa(1,:)=',zthetaa(1,:)
             WRITE(numout,*) 'zthetaa_pft(1,:,:)=',zthetaa_pft(1,:,:)
          ENDIF

          ! compute interface average temperatures (K):
          ! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
          !
          ztemp(ji,0)      = snowtemp(ji,1)
          DO jj=1,nsnow-1
             ztemp(ji,jj)  = (zsnowdz(ji,jj)  *snowtemp(ji,jj)   +             &
                                 zsnowdz(ji,jj+1)*snowtemp(ji,jj+1))/zdz(ji,jj)
          ENDDO
          ztemp(ji,nsnow) = snowtemp(ji,nsnow)

          !yx:+++++
          DO jv=1,nvm
             ztemp_pft(ji,0,jv)      = snowtemp_pft(ji,1,jv)
             DO jj=1,nsnow-1
                ztemp_pft(ji,jj,jv)  = (zsnowdz_pft(ji,jj,jv)  *snowtemp_pft(ji,jj,jv)   +             &
                                    zsnowdz_pft(ji,jj+1,jv)*snowtemp_pft(ji,jj+1,jv))/zdz_pft(ji,jj,jv)
             ENDDO
             ztemp_pft(ji,nsnow,jv) = snowtemp_pft(ji,nsnow,jv)
          ENDDO
          !yx:-----
          IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) ) THEN
             WRITE(numout,*) 'ztemp(1,:)=',ztemp(1,:)
             WRITE(numout,*) 'ztemp_pft(1,:,:)=',ztemp_pft(1,:,:)
          ENDIF

!
          ! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
          ! compute variation of saturation vapor pressure with temperature
          ! for solid and liquid phases:
          ! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
          zexpo(ji,:)    = chalsu0/(xrv*ztemp(ji,:))
          zckt_ice(ji,:) = (zc1_ice/ztemp(ji,:)**2)*(zexpo(ji,:) - 1.0)*EXP(-zexpo(ji,:))
          zexpo_pft(ji,:,:)    = chalsu0/(xrv*ztemp_pft(ji,:,:)) !yx:+-
          zckt_ice_pft(ji,:,:) = (zc1_ice/ztemp_pft(ji,:,:)**2)*(zexpo_pft(ji,:,:) - 1.0)*EXP(-zexpo_pft(ji,:,:)) !yx:+-
          IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) ) THEN
             WRITE(numout,*) 'zexpo(1,:)=',zexpo(1,:)
             WRITE(numout,*) 'zexpo_pft(1,:,:)=',zexpo_pft(1,:,:)
             WRITE(numout,*) 'zckt_ice(1,:)=',zckt_ice(1,:)
             WRITE(numout,*) 'zckt_ice_pft(1,:,:)=',zckt_ice_pft(1,:,:)
          ENDIF

          !
          zexpo(ji,:)    = chalev0/(xrv*ztemp(ji,:))
          zckt_liq(ji,:) = (zc1_liq/ztemp(ji,:)**2)*(zexpo(ji,:) - 1.0)*EXP(-zexpo(ji,:))
          !
          zexpo_pft(ji,:,:)    = chalev0/(xrv*ztemp_pft(ji,:,:)) !yx:+-
          zckt_liq_pft(ji,:,:) = (zc1_liq/ztemp_pft(ji,:,:)**2)*(zexpo_pft(ji,:,:) - 1.0)*EXP(-zexpo_pft(ji,:,:)) !yx:+-
          IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) ) THEN
             WRITE(numout,*) 'zexpo(1,:)=',zexpo(1,:)
             WRITE(numout,*) 'zexpo_pft(1,:,:)=',zexpo_pft(1,:,:)
             WRITE(numout,*) 'zckt_liq(1,:)=',zckt_liq(1,:)
             WRITE(numout,*) 'zckt_liq_pft(1,:,:)=',zckt_liq_pft(1,:,:)
          ENDIF

          !
          ! compute the weighted ice/liquid total variation (N m-2 K):
          !
          zfrac(ji,:)    = MIN(1.0, zthetaa(ji,:)/ztheta_crit)
          zckt(ji,:)     = zfrac(ji,:)*zckt_liq(ji,:) + (1.0 - zfrac(ji,:))*zckt_ice(ji,:)
          zfrac_pft(ji,:,:)    = MIN(1.0, zthetaa_pft(ji,:,:)/ztheta_crit) !yx:+-
          zckt_pft(ji,:,:)     = zfrac_pft(ji,:,:)*zckt_liq_pft(ji,:,:) + (1.0 - zfrac_pft(ji,:,:))*zckt_ice_pft(ji,:,:) !yx:+-
          IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) ) THEN
             WRITE(numout,*) 'zfrac(1,:)=',zfrac(1,:)
             WRITE(numout,*) 'zfrac_pft(1,:,:)=',zfrac_pft(1,:,:)
             WRITE(numout,*) 'zckt(1,:)=',zckt(1,:)
             WRITE(numout,*) 'zckt_pft(1,:,:)=',zckt_pft(1,:,:)
          ENDIF

          ! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
          ! Compute effective diffusion coefficient (m2 s-1):
          ! -diffudivity relative to a reference diffusion at 1000 mb and freezing point
          !  multiplied by phase energy coefficient
          ! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
          !
          DO jj=0,nsnow
             zdiff(ji,jj) = zdeos*(xp00/(pb(ji)*100.))*((ztemp(ji,jj)/tp_00)**6)*zckt(ji,jj)
          ENDDO 
          !yx:+++++
          DO jv=1,nvm
             DO jj=0,nsnow
                zdiff_pft(ji,jj,jv) = zdeos*(xp00/(pb(ji)*100.))*((ztemp_pft(ji,jj,jv)/tp_00)**6)*zckt_pft(ji,jj,jv)
             ENDDO
          ENDDO
          !yx:-----
          IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) ) THEN
             WRITE(numout,*) 'zdiff(1,:)=',zdiff(1,:)
             WRITE(numout,*) 'zdiff_pft(1,:,:)=',zdiff_pft(1,:,:)
          ENDIF

          ! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
          ! Temperature gradient (K m-1):

          ztgrad(ji,0)      = 0.0 ! uppermost layer-mean and surface T's are assumed to be equal
          DO jj=1,nsnow-1
             ztgrad(ji,jj)  = 2*(snowtemp(ji,jj)-snowtemp(ji,jj+1))/MAX(zdzmin, zdz(ji,jj))
          ENDDO

          !yx:+++++
          DO jv=1,nvm
             ztgrad_pft(ji,0,jv)      = 0.0 ! uppermost layer-mean and surface T's are assumed to be equal
             DO jj=1,nsnow-1
                ztgrad_pft(ji,jj,jv)  = 2*(snowtemp_pft(ji,jj,jv)-snowtemp_pft(ji,jj+1,jv))/MAX(zdzmin, zdz_pft(ji,jj,jv))
             ENDDO
          ENDDO
          !yx:-----

          !
          ! assume at base of snow, temperature is in equilibrium with soil
          ! (but obviously must be at or below freezing point!)
          !
          ztgrad(ji,nsnow) = 2*(snowtemp(ji,nsnow) - MIN(tp_00, gtemp(ji)))/MAX(zdzmin, zdz(ji,nsnow))
          ztgrad_pft(ji,nsnow,:) = 2*(snowtemp_pft(ji,nsnow,:) - MIN(tp_00, gtemp_pft(ji,:)))/MAX(zdzmin, zdz_pft(ji,nsnow,:)) !yx:+-
          IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) ) THEN
             WRITE(numout,*) 'ztgrad(1,:)=',ztgrad(1,:)
             WRITE(numout,*) 'ztgrad_pft(1,:,:)=',ztgrad_pft(1,:,:)
          ENDIF

          ! prognostic grain size (m) equation:
          !-------------------------------------------------------------------
          ! first compute the minimum grain size (m):
          !
          zrhomin(ji,:)     = xrhosmin
          zgrainmin(ji,:)   = snow3lgrain_1d(zrhomin(ji,:))
          !yx:+++++
          DO jv=1,nvm
             zrhomin_pft(ji,:,jv)     = xrhosmin
             zgrainmin_pft(ji,:,jv)   = snow3lgrain_1d(zrhomin_pft(ji,:,jv))
          ENDDO
          !yx:-----
          IF ( writeforcheck_explicitsnow .AND. (ji .EQ. 1) ) THEN
             WRITE(numout,*) 'zrhomin(1,:)=',zrhomin(1,:)
             WRITE(numout,*) 'zrhomin_pft(1,:,:)=',zrhomin_pft(1,:,:)
             WRITE(numout,*) 'zgrainmin(1,:)=',zgrainmin(1,:)
             WRITE(numout,*) 'zgrainmin_pft(1,:,:)=',zgrainmin_pft(1,:,:)
          ENDIF

          ! dry snow:
          ! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
          !
          DO jj=1,nsnow

             IF(ztheta(ji,jj) == 0.0) THEN

              ! only allow growth due to vapor flux INTO layer: 
              ! aab add sublimation(only condensation) as upper BC...?

                snowgrain(ji,jj)  = snowgrain(ji,jj) +                                      &
                                       (dt_sechiba*zg1/MAX(zgrainmin(ji,jj),snowgrain(ji,jj)))*      &
                                       ( zdiff(ji,jj-1)*MAX(0.0,ztgrad(ji,jj-1)) -                &
                                       zdiff(ji,jj)  *MIN(0.0,ztgrad(ji,jj)) )
             ELSE
 
              ! wet snow
              ! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
              !
                snowgrain(ji,jj)  = snowgrain(ji,jj) +                                      &
                                       (dt_sechiba*zg2/MAX(zgrainmin(ji,jj),snowgrain(ji,jj)))*      &
                                       MIN(ztheta_crit_w, ztheta(ji,jj) + ztheta_w)
             END IF

             !yx:+++++
             DO jv=1,nvm
                IF(ztheta_pft(ji,jj,jv) == 0.0) THEN

                 ! only allow growth due to vapor flux INTO layer:
                 ! aab add sublimation(only condensation) as upper BC...?

                 snowgrain_pft(ji,jj,jv)  = snowgrain_pft(ji,jj,jv) +                                      &
                                          (dt_sechiba*zg1/MAX(zgrainmin_pft(ji,jj,jv),snowgrain_pft(ji,jj,jv)))*      &
                                          ( zdiff_pft(ji,jj-1,jv)*MAX(0.0,ztgrad_pft(ji,jj-1,jv)) -                &
                                          zdiff_pft(ji,jj,jv)  *MIN(0.0,ztgrad_pft(ji,jj,jv)) )
                ELSE

                 ! wet snow
                 ! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                 !
                 snowgrain_pft(ji,jj,jv)  = snowgrain_pft(ji,jj,jv) +                                      &
                                          (dt_sechiba*zg2/MAX(zgrainmin_pft(ji,jj,jv),snowgrain_pft(ji,jj,jv)))*      &
                                          MIN(ztheta_crit_w, ztheta_pft(ji,jj,jv) + ztheta_w)
                END IF
             ENDDO
             !yx:-----

          ENDDO

      ENDDO


    END SUBROUTINE explicitsnow_grain

!!
!================================================================================================================================
!! SUBROUTINE   : explicitsnow_compactn
!!
!>\BRIEF        Compute Compaction/Settling
!!                
!! DESCRIPTION  : 
!!     Snow compaction due to overburden and settling.
!!     Mass is unchanged: layer thickness is reduced
!!     in proportion to density increases. Method
!!     of Anderson (1976): see Loth and Graf, 1993,
!!     J. of Geophys. Res., 98, 10,451-10,464.
!!
!! RECENT CHANGE(S) : None 
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : Loth and Graf (1993), Mellor (1964) and Anderson (1976)
!!
!! FLOWCHART    : None
!! \n
!_
!================================================================================================================================


   SUBROUTINE explicitsnow_compactn(kjpindex,snowtemp,snowrho,snowdz,&
              snowtemp_pft, snowrho_pft, snowdz_pft) !yx:+-

         !! 0.1 Input variables

         INTEGER(i_std),INTENT(in)                                 :: kjpindex         !! Domain size
         REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(in)          :: snowtemp         !! Snow temperature
         REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(in)      :: snowtemp_pft     !! Snow temperature !yx:+-

         !! 0.2 Output variables

         !! 0.3 Modified variables

         REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(inout)       :: snowrho          !! Snow density (Kg/m^3)
         REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(inout)   :: snowrho_pft      !! Snow density (Kg/m^3) !yx:+-
         REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(inout)       :: snowdz           !! Snow depth
         REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(inout)   :: snowdz_pft       !! Snow depth !yx:+-

         !! 0.4 Local variables

         REAL(r_std),DIMENSION(kjpindex,nsnow)                     :: zwsnowdz,zsmass,zsnowrho2,zviscocity,zsettle 
         REAL(r_std),DIMENSION(kjpindex,nsnow,nvm)                 :: zwsnowdz_pft,zsmass_pft,zsnowrho2_pft,zviscocity_pft,zsettle_pft !yx:+-
         REAL(r_std),DIMENSION(kjpindex)                           :: zsmassc          !! cummulative snow mass (kg/m2)
         REAL(r_std),DIMENSION(kjpindex,nvm)                       :: zsmassc_pft          !! cummulative snow mass (kg/m2) !yx:+-
         !REAL(r_std),DIMENSION(kjpindex)                           :: snowdepth_crit !yx:+- commented, not used
         INTEGER(i_std)                                            :: ji,jj,jv !yx:+-

        !! 1. initialize

        zsnowrho2  = snowrho
        zsettle(:,:)    = ZSNOWCMPCT_ACM
        zviscocity(:,:) = ZSNOWCMPCT_V0

        !yx:+++++
        zsnowrho2_pft  = snowrho_pft
        zsettle_pft(:,:,:)    = ZSNOWCMPCT_ACM
        zviscocity_pft(:,:,:) = ZSNOWCMPCT_V0
        !yx:-----

        !! 2. Calculating Cumulative snow mass (kg/m2):

        DO ji=1, kjpindex


            IF (SUM(snowdz(ji,:)) .GT. 0.0) THEN 

              zwsnowdz(ji,:)= snowdz(ji,:)*snowrho(ji,:)

              zsmassc (ji)= 0.0

              DO jj=1,nsnow
                 zsmass(ji,jj)   = zsmassc(ji) + zwsnowdz(ji,jj)
                 zsmassc(ji)      = zsmassc(ji) + zwsnowdz(ji,jj)
              ENDDO


              !! 3. Computing compaction/Settling
              ! ----------------------
              ! Compaction/settling if density below upper limit
              ! (compaction is generally quite small above ~ 500 kg m-3):
              !
              DO jj=1,nsnow
                 IF (snowrho(ji,jj) .LT. xrhosmax) THEN
            
              !
              ! First calculate settling due to freshly fallen snow: (NOTE:bug here for the snow temperature profile)
              !
             
              zsettle(ji,jj)     = ZSNOWCMPCT_ACM*EXP(                                      &
                                     -ZSNOWCMPCT_BCM*(tp_00-MIN(tp_00,snowtemp(ji,jj)))                      &
                                     -ZSNOWCMPCT_CCM*MAX(0.0,                                  &
                                      snowrho(ji,jj)-ZSNOWCMPCT_RHOD))
              !
              ! Snow viscocity:
              !
              zviscocity(ji,jj)   = ZSNOWCMPCT_V0*EXP( ZSNOWCMPCT_VT*(tp_00-MIN(tp_00,snowtemp(ji,jj))) +        &
                                  ZSNOWCMPCT_VR*snowrho(ji,jj) )

              ! Calculate snow density: compaction from weight/over-burden
              ! Anderson 1976 method:
              zsnowrho2(ji,jj)    = snowrho(ji,jj) + snowrho(ji,jj)*dt_sechiba*(          &
                                   (cte_grav*zsmass(ji,jj)/zviscocity(ji,jj))                 &
                                    + zsettle(ji,jj) )
              ! Conserve mass by decreasing grid thicknesses in response
              ! to density increases
              !
              snowdz(ji,jj)  = snowdz(ji,jj)*(snowrho(ji,jj)/zsnowrho2(ji,jj))

                 ENDIF
              ENDDO

              ! Update density (kg m-3):
              snowrho(ji,:) = zsnowrho2(ji,:)

            ENDIF

        ENDDO


        !yx:+++++
        DO ji=1, kjpindex
           DO jv=1,nvm

              IF (SUM(snowdz_pft(ji,:,jv)) .GT. 0.0) THEN

                 zwsnowdz_pft(ji,:,jv)= snowdz_pft(ji,:,jv)*snowrho_pft(ji,:,jv)

                 zsmassc_pft (ji,jv)= 0.0

                 DO jj=1,nsnow
                    zsmass_pft(ji,jj,jv)   = zsmassc_pft(ji,jv) + zwsnowdz_pft(ji,jj,jv)
                    zsmassc_pft(ji,jv)      = zsmassc_pft(ji,jv) + zwsnowdz_pft(ji,jj,jv)
                 ENDDO


                 !! 3. Computing compaction/Settling
                 ! ----------------------
                 ! Compaction/settling if density below upper limit
                 ! (compaction is generally quite small above ~ 500 kg m-3):
                 !
                 DO jj=1,nsnow
                    IF (snowrho_pft(ji,jj,jv) .LT. xrhosmax) THEN

                       !
                       ! First calculate settling due to freshly fallen snow: (NOTE:bug here for the snow temperature profile)
                       !

                       zsettle_pft(ji,jj,jv)     = ZSNOWCMPCT_ACM*EXP(                                      &
                                              -ZSNOWCMPCT_BCM*(tp_00-MIN(tp_00,snowtemp_pft(ji,jj,jv)))                      &
                                              -ZSNOWCMPCT_CCM*MAX(0.0,                                  &
                                               snowrho_pft(ji,jj,jv)-ZSNOWCMPCT_RHOD))
                       !
                       ! Snow viscocity:
                       !
                       zviscocity_pft(ji,jj,jv)   = ZSNOWCMPCT_V0*EXP( ZSNOWCMPCT_VT*(tp_00-MIN(tp_00,snowtemp_pft(ji,jj,jv))) +        &
                                           ZSNOWCMPCT_VR*snowrho_pft(ji,jj,jv) )

                       ! Calculate snow density: compaction from weight/over-burden
                       ! Anderson 1976 method:
                       zsnowrho2_pft(ji,jj,jv)    = snowrho_pft(ji,jj,jv) + snowrho_pft(ji,jj,jv)*dt_sechiba*(          &
                                           (cte_grav*zsmass_pft(ji,jj,jv)/zviscocity_pft(ji,jj,jv))                 &
                                           + zsettle_pft(ji,jj,jv) )
                       ! Conserve mass by decreasing grid thicknesses in response
                       ! to density increases
                       !
                       snowdz_pft(ji,jj,jv)  = snowdz_pft(ji,jj,jv)*(snowrho_pft(ji,jj,jv)/zsnowrho2_pft(ji,jj,jv))

                    ENDIF
                 ENDDO
  
                 ! Update density (kg m-3):
                 snowrho_pft(ji,:,jv) = zsnowrho2_pft(ji,:,jv)

              ENDIF

           ENDDO
        ENDDO
        !yx:-----

      END SUBROUTINE explicitsnow_compactn

!!
!================================================================================================================================
!! SUBROUTINE   : explicitsnow_transf
!!
!>\BRIEF        Computing snow mass and heat redistribution due to grid thickness configuration resetting
!!                
!! DESCRIPTION  : Snow mass and heat redistibution due to grid thickness
!!                configuration resetting. Total mass and heat content
!!                of the overall snowpack unchanged/conserved within this routine. 
!! RECENT CHANGE(S) : None 
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : 
!!
!! FLOWCHART    : None
!! \n
!_
!================================================================================================================================

   SUBROUTINE explicitsnow_transf(kjpindex,snowdz_old,snowdz,snowrho,snowheat,snowgrain,&
              snowdz_old_pft, snowrho_pft, snowgrain_pft, snowdz_pft, snowheat_pft) !yx:+-

     !! 0.1 Input variables

     INTEGER(i_std),INTENT(in)                                           :: kjpindex         !! Domain size
     REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(in)                    :: snowdz_old       !! Snow depth at the previous time step 
     REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(in)                :: snowdz_old_pft   !! Snow depth at the previous time step !yx:+-

     !! 0.2 Output variables

     !! 0.3 Modified variables

     REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(inout)                 :: snowrho          !! Snow density (Kg/m^3)
     REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(inout)             :: snowrho_pft      !! Snow density (Kg/m^3) !yx:+-
     REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(inout)                 :: snowgrain        !! Snow grain size (m)
     REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(inout)             :: snowgrain_pft    !! Snow grain size (m) !yx:+-
     REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(inout)                 :: snowdz           !! Snow depth (m)
     REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(inout)             :: snowdz_pft       !! Snow depth (m) !yx:+-
     REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(inout)                 :: snowheat         !! Snow heat content/enthalpy (J/m2)
     REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(inout)             :: snowheat_pft     !! Snow heat content/enthalpy (J/m2) !yx:+-
                                                                        
     !! 0.4 Local varibles                                              
      
     REAL(r_std),DIMENSION(kjpindex,0:nsnow)                             :: zsnowzo
     REAL(r_std),DIMENSION(kjpindex,0:nsnow,nvm)                         :: zsnowzo_pft !yx:+-
     REAL(r_std),DIMENSION(kjpindex,0:nsnow)                             :: zsnowzn
     REAL(r_std),DIMENSION(kjpindex,0:nsnow,nvm)                         :: zsnowzn_pft !yx:+-
     REAL(r_std),DIMENSION(kjpindex,nsnow)                               :: zsnowddz 
     REAL(r_std),DIMENSION(kjpindex,nsnow,nvm)                           :: zsnowddz_pft !yx:+-
     REAL(r_std),DIMENSION(kjpindex,nsnow)                               :: zdelta
     REAL(r_std),DIMENSION(kjpindex,nsnow,nvm)                           :: zdelta_pft !yx:+-
     REAL(r_std),DIMENSION(kjpindex,nsnow)                               :: zsnowrhon,zsnowheatn,zsnowgrainn
     REAL(r_std),DIMENSION(kjpindex,nsnow,nvm)                           :: zsnowrhon_pft,zsnowheatn_pft,zsnowgrainn_pft !yx:+-
     REAL(r_std),DIMENSION(kjpindex)                                     :: zsnowmix_delta
     REAL(r_std),DIMENSION(kjpindex,nvm)                                 :: zsnowmix_delta_pft !yx:+-
     REAL(r_std),DIMENSION(kjpindex)                                     :: zsumheat, zsumswe, zsumgrain
     REAL(r_std),DIMENSION(kjpindex,nvm)                                 :: zsumheat_pft, zsumswe_pft, zsumgrain_pft !yx:+-
     INTEGER(i_std),DIMENSION(nsnow,2)                                   :: locflag
     INTEGER(i_std),DIMENSION(nsnow,2,nvm)                               :: locflag_pft !yx:+-
     REAL(r_std)                                                         :: psnow
     REAL(r_std),DIMENSION(nvm)                                          :: psnow_pft !yx:+-
     INTEGER(i_std)                                                      :: ji,jj,jjj,jv !yx:+-

     ! Initialization
     zsnowzo(:,:)      = 0.0
     zsnowzn(:,:)      = 0.0
     zsnowddz(:,:)     = 0.0
     zdelta(:,:)       = 0.0
     zsnowrhon(:,:)    = 0.0
     zsnowheatn(:,:)   = 0.0
     zsnowgrainn(:,:)  = 0.0
 
     !yx:+++++
     zsnowzo_pft(:,:,:)      = 0.0
     zsnowzn_pft(:,:,:)      = 0.0
     zsnowddz_pft(:,:,:)     = 0.0
     zdelta_pft(:,:,:)       = 0.0
     zsnowrhon_pft(:,:,:)    = 0.0
     zsnowheatn_pft(:,:,:)   = 0.0
     zsnowgrainn_pft(:,:,:)  = 0.0
     !yx:-----

     zsumheat(:)       = 0.0
     zsumswe(:)        = 0.0
     zsumgrain(:)      = 0.0
     zsnowmix_delta(:) = 0.0
     locflag(:,:)      = 0

     !yx:+++++
     zsumheat_pft(:,:)       = 0.0
     zsumswe_pft(:,:)        = 0.0
     zsumgrain_pft(:,:)      = 0.0
     zsnowmix_delta_pft(:,:) = 0.0
     locflag_pft(:,:,:)      = 0
     !yx:-----

     DO ji=1, kjpindex 
 

        psnow = SUM(snowdz(ji,:))

        IF (psnow .GE. xsnowcritd .AND. snowdz_old(ji,1) .NE. 0 .AND. snowdz_old(ji,2) .NE. 0 .AND. snowdz_old(ji,3) .NE. 0) THEN
          !
          zsnowzo(ji,0)     = 0.
          zsnowzn(ji,0)     = 0.
          zsnowzo(ji,1)     = snowdz_old(ji,1)
          zsnowzn(ji,1)     = snowdz(ji,1)
          !
          DO jj=2,nsnow
             zsnowzo(ji,jj) = zsnowzo(ji,jj-1) + snowdz_old(ji,jj)
             zsnowzn(ji,jj) = zsnowzn(ji,jj-1) + snowdz(ji,jj)
          ENDDO

          DO jj=1,nsnow

             IF (jj .EQ. 1) THEN 
                locflag(jj,1)=1 
             ELSE       
                DO jjj=nsnow,1,-1

                   !upper bound of the snow layer
                   IF (zsnowzn(ji,jj-1) .LE. zsnowzo(ji,jjj)) THEN
                      locflag(jj,1) = jjj
                   ENDIF
                ENDDO
	     ENDIF

             IF (jj .EQ. nsnow) THEN 
                locflag(jj,2)=nsnow 
             ELSE       
                DO jjj=nsnow,1,-1

                   !lower bound of the snow layer
                   IF (zsnowzn(ji,jj)   .LE. zsnowzo(ji,jjj)) THEN
                      locflag(jj,2) = jjj
                   ENDIF
                ENDDO
             ENDIF

             !to interpolate
             ! when heavy snow occurred
             IF (locflag(jj,1) .EQ. locflag(jj,2)) THEN 

                zsnowrhon(ji,jj) = snowrho(ji,locflag(jj,1))

                zsnowheatn(ji,jj) = snowheat(ji,locflag(jj,1))*snowdz(ji,jj)/snowdz_old(ji,locflag(jj,1)) 

                zsnowgrainn(ji,jj) = snowgrain(ji,locflag(jj,1)) 

             ELSE 

                !snow density
                zsnowrhon(ji,jj) = snowrho(ji,locflag(jj,1)) * &
                                      (zsnowzo(ji,locflag(jj,1))-zsnowzn(ji,jj-1)) + &
                                      snowrho(ji,locflag(jj,2)) * &
                                      (zsnowzn(ji,jj)-zsnowzo(ji,locflag(jj,2)-1))

                DO jjj=locflag(jj,1),locflag(jj,2)-1
                   zsnowrhon(ji,jj)=zsnowrhon(ji,jj) + &
                   (jjj-locflag(jj,1))*snowrho(ji,jjj)*snowdz_old(ji,jjj)  
                ENDDO

                zsnowrhon(ji,jj) = zsnowrhon(ji,jj) / snowdz(ji,jj)


                !snow heat
                IF (snowdz_old(ji,locflag(jj,1)) .GT. 0.0) THEN

                   zsnowheatn(ji,jj) = snowheat(ji,locflag(jj,1)) * &
                                 (zsnowzo(ji,locflag(jj,1))-zsnowzn(ji,jj-1))/snowdz_old(ji,locflag(jj,1)) + &
                                       snowheat(ji,locflag(jj,2)) * &
                                 (zsnowzn(ji,jj)-zsnowzo(ji,locflag(jj,2)-1))/snowdz_old(ji,locflag(jj,2))
	        ELSE
                   zsnowheatn(ji,jj) = snowheat(ji,locflag(jj,2))/snowdz_old(ji,locflag(jj,2)) * &
                        (zsnowzn(ji,locflag(jj,1))-zsnowzo(ji,locflag(jj,1)))
		ENDIF

                DO jjj=locflag(jj,1),locflag(jj,2)-1
                   zsnowheatn(ji,jj)=zsnowheatn(ji,jj) + &
                                        (jjj-locflag(jj,1))*snowheat(ji,jjj)
                ENDDO


                !snow grain
                zsnowgrainn(ji,jj) = snowgrain(ji,locflag(jj,1)) * &
                                      (zsnowzo(ji,locflag(jj,1))-zsnowzn(ji,jj-1)) + &
                                      snowgrain(ji,locflag(jj,2)) * &
                                      (zsnowzn(ji,jj)-zsnowzo(ji,locflag(jj,2)-1))

                DO jjj=locflag(jj,1),locflag(jj,2)-1
                   zsnowgrainn(ji,jj)=zsnowgrainn(ji,jj) + &
                   (jjj-locflag(jj,1))*snowgrain(ji,jjj)*snowdz_old(ji,jjj)
                ENDDO

                zsnowgrainn(ji,jj) = zsnowgrainn(ji,jj) / snowdz(ji,jj)

             ENDIF
                        
          ENDDO

          snowrho(ji,:)    = zsnowrhon(ji,:)
          snowheat(ji,:)   = zsnowheatn(ji,:)
          snowgrain(ji,:)  = zsnowgrainn(ji,:)

        ENDIF

          ! Vanishing or very thin snowpack check:
          ! -----------------------------------------
          !
          ! NOTE: ONLY for very shallow snowpacks, mix properties (homogeneous):
          ! this avoids problems related to heat and mass exchange for
          ! thin layers during heavy snowfall or signifigant melt: one
          ! new/old layer can exceed the thickness of several old/new layers.
          ! Therefore, mix (conservative):
          !
          ! modified by Tao Wang
        IF (psnow > 0 .AND. (psnow < xsnowcritd .OR. snowdz_old(ji,1) &
            & .eq. 0 .OR. snowdz_old(ji,2) .eq. 0 .OR. snowdz_old(ji,3) .eq. 0)) THEN
                zsumheat(ji) = SUM(snowheat(ji,:))
                zsumswe(ji)  = SUM(snowrho(ji,:)*snowdz_old(ji,:))
                zsumgrain(ji)= SUM(snowgrain(ji,:)*snowdz_old(ji,:))
                zsnowmix_delta(ji) = 1.0
                DO jj=1,nsnow
                   zsnowheatn(ji,jj)  = zsnowmix_delta(ji)*(zsumheat(ji)/nsnow)
                   snowdz(ji,jj)    = zsnowmix_delta(ji)*(psnow/nsnow) 
                   zsnowrhon(ji,jj)   = zsnowmix_delta(ji)*(zsumswe(ji)/psnow)
                   zsnowgrainn(ji,jj) = zsnowmix_delta(ji)*(zsumgrain(ji)/psnow)
                ENDDO
          ! Update mass (density and thickness), heat and grain size:
          ! ------------------------------------------------------------
          !
          snowrho(ji,:)    = zsnowrhon(ji,:)
          snowheat(ji,:)   = zsnowheatn(ji,:)
          snowgrain(ji,:)  = zsnowgrainn(ji,:)

        ENDIF

     ENDDO


     !yx:+++++
     DO ji=1, kjpindex
        DO jv=1,nvm

           psnow_pft(jv) = SUM(snowdz_pft(ji,:,jv))

           IF (psnow_pft(jv) .GE. xsnowcritd .AND. snowdz_old_pft(ji,1,jv) .NE. 0 .AND. snowdz_old_pft(ji,2,jv) .NE. 0 .AND. snowdz_old_pft(ji,3,jv) .NE. 0) THEN
              !
              zsnowzo_pft(ji,0,jv)     = 0.
              zsnowzn_pft(ji,0,jv)     = 0.
              zsnowzo_pft(ji,1,jv)     = snowdz_old_pft(ji,1,jv)
              zsnowzn_pft(ji,1,jv)     = snowdz_pft(ji,1,jv)
              !
              DO jj=2,nsnow
                 zsnowzo_pft(ji,jj,jv) = zsnowzo_pft(ji,jj-1,jv) + snowdz_old_pft(ji,jj,jv)
                 zsnowzn_pft(ji,jj,jv) = zsnowzn_pft(ji,jj-1,jv) + snowdz_pft(ji,jj,jv)
              ENDDO

              DO jj=1,nsnow

                 IF (jj .EQ. 1) THEN
                    locflag_pft(jj,1,jv)=1
                 ELSE
                    DO jjj=nsnow,1,-1
 
                       !upper bound of the snow layer
                       IF (zsnowzn_pft(ji,jj-1,jv) .LE. zsnowzo_pft(ji,jjj,jv)) THEN
                          locflag_pft(jj,1,jv) = jjj
                       ENDIF
                    ENDDO
                 ENDIF

                 IF (jj .EQ. nsnow) THEN
                    locflag_pft(jj,2,jv)=nsnow
                 ELSE
                    DO jjj=nsnow,1,-1

                       !lower bound of the snow layer
                       IF (zsnowzn_pft(ji,jj,jv)   .LE. zsnowzo_pft(ji,jjj,jv)) THEN
                          locflag_pft(jj,2,jv) = jjj
                       ENDIF
                    ENDDO
                 ENDIF

                 !to interpolate
                 ! when heavy snow occurred
                 IF (locflag_pft(jj,1,jv) .EQ. locflag_pft(jj,2,jv)) THEN

                    zsnowrhon_pft(ji,jj,jv) = snowrho_pft(ji,locflag_pft(jj,1,jv),jv)
  
                    zsnowheatn_pft(ji,jj,jv) = snowheat_pft(ji,locflag_pft(jj,1,jv),jv)*snowdz_pft(ji,jj,jv)/snowdz_old_pft(ji,locflag_pft(jj,1,jv),jv)

                    zsnowgrainn_pft(ji,jj,jv) = snowgrain_pft(ji,locflag_pft(jj,1,jv),jv)

                 ELSE

                    !snow density
                    zsnowrhon_pft(ji,jj,jv) = snowrho_pft(ji,locflag_pft(jj,1,jv),jv) * &
                                          (zsnowzo_pft(ji,locflag_pft(jj,1,jv),jv)-zsnowzn_pft(ji,jj-1,jv)) + &
                                          snowrho_pft(ji,locflag_pft(jj,2,jv),jv) * &
                                          (zsnowzn_pft(ji,jj,jv)-zsnowzo_pft(ji,locflag_pft(jj,2,jv)-1,jv))

                    DO jjj=locflag_pft(jj,1,jv),locflag_pft(jj,2,jv)-1
                       zsnowrhon_pft(ji,jj,jv)=zsnowrhon_pft(ji,jj,jv) + &
                       (jjj-locflag_pft(jj,1,jv))*snowrho_pft(ji,jjj,jv)*snowdz_old_pft(ji,jjj,jv)
                    ENDDO

                    zsnowrhon_pft(ji,jj,jv) = zsnowrhon_pft(ji,jj,jv) / snowdz_pft(ji,jj,jv)

                    !snow heat
                    IF (snowdz_old_pft(ji,locflag_pft(jj,1,jv),jv) .GT. 0.0) THEN

                       zsnowheatn_pft(ji,jj,jv) = snowheat_pft(ji,locflag_pft(jj,1,jv),jv) * &
                                     (zsnowzo_pft(ji,locflag_pft(jj,1,jv),jv)-zsnowzn_pft(ji,jj-1,jv))/snowdz_old_pft(ji,locflag_pft(jj,1,jv),jv) + &
                                           snowheat_pft(ji,locflag_pft(jj,2,jv),jv) * &
                                     (zsnowzn_pft(ji,jj,jv)-zsnowzo_pft(ji,locflag_pft(jj,2,jv)-1,jv))/snowdz_old_pft(ji,locflag_pft(jj,2,jv),jv)
                    ELSE
                       zsnowheatn_pft(ji,jj,jv) = snowheat_pft(ji,locflag_pft(jj,2,jv),jv)/snowdz_old_pft(ji,locflag_pft(jj,2,jv),jv) * &
                              (zsnowzn_pft(ji,locflag_pft(jj,1,jv),jv)-zsnowzo_pft(ji,locflag_pft(jj,1,jv),jv))
                    ENDIF

                    DO jjj=locflag_pft(jj,1,jv),locflag_pft(jj,2,jv)-1
                       zsnowheatn_pft(ji,jj,jv)=zsnowheatn_pft(ji,jj,jv) + &
                                            (jjj-locflag_pft(jj,1,jv))*snowheat_pft(ji,jjj,jv)
                    ENDDO


                    !snow grain
                    zsnowgrainn_pft(ji,jj,jv) = snowgrain_pft(ji,locflag_pft(jj,1,jv),jv) * &
                                          (zsnowzo_pft(ji,locflag_pft(jj,1,jv),jv)-zsnowzn_pft(ji,jj-1,jv)) + &
                                          snowgrain_pft(ji,locflag_pft(jj,2,jv),jv) * &
                                          (zsnowzn_pft(ji,jj,jv)-zsnowzo_pft(ji,locflag_pft(jj,2,jv)-1,jv))
                    DO jjj=locflag_pft(jj,1,jv),locflag_pft(jj,2,jv)-1
                       zsnowgrainn_pft(ji,jj,jv)=zsnowgrainn_pft(ji,jj,jv) + &
                       (jjj-locflag_pft(jj,1,jv))*snowgrain_pft(ji,jjj,jv)*snowdz_old_pft(ji,jjj,jv)
                    ENDDO

                    zsnowgrainn_pft(ji,jj,jv) = zsnowgrainn_pft(ji,jj,jv) / snowdz_pft(ji,jj,jv)
 
                 ENDIF

              ENDDO
  
              snowrho_pft(ji,:,jv)    = zsnowrhon_pft(ji,:,jv)
              snowheat_pft(ji,:,jv)   = zsnowheatn_pft(ji,:,jv)
              snowgrain_pft(ji,:,jv)  = zsnowgrainn_pft(ji,:,jv)
           ENDIF

           ! Vanishing or very thin snowpack check:
           ! -----------------------------------------
           !
           ! NOTE: ONLY for very shallow snowpacks, mix properties (homogeneous):
           ! this avoids problems related to heat and mass exchange for
           ! thin layers during heavy snowfall or signifigant melt: one
           ! new/old layer can exceed the thickness of several old/new layers.
           ! Therefore, mix (conservative):
           !
           ! modified by Tao Wang
           IF (psnow_pft(jv) > 0 .AND. (psnow_pft(jv) < xsnowcritd .OR. snowdz_old_pft(ji,1,jv) &
                 & .eq. 0 .OR. snowdz_old_pft(ji,2,jv) .eq. 0 .OR. snowdz_old_pft(ji,3,jv) .eq. 0)) THEN
                     zsumheat_pft(ji,jv) = SUM(snowheat_pft(ji,:,jv))
                     zsumswe_pft(ji,jv)  = SUM(snowrho_pft(ji,:,jv)*snowdz_old_pft(ji,:,jv))
                     zsumgrain_pft(ji,jv)= SUM(snowgrain_pft(ji,:,jv)*snowdz_old_pft(ji,:,jv))
                     zsnowmix_delta_pft(ji,jv) = 1.0
                     DO jj=1,nsnow
                        zsnowheatn_pft(ji,jj,jv)  = zsnowmix_delta_pft(ji,jv)*(zsumheat_pft(ji,jv)/nsnow)
                        snowdz_pft(ji,jj,jv)    = zsnowmix_delta_pft(ji,jv)*(psnow_pft(jv)/nsnow)
                        zsnowrhon_pft(ji,jj,jv)   = zsnowmix_delta_pft(ji,jv)*(zsumswe_pft(ji,jv)/psnow_pft(jv))
                        zsnowgrainn_pft(ji,jj,jv) = zsnowmix_delta_pft(ji,jv)*(zsumgrain_pft(ji,jv)/psnow_pft(jv))
                     ENDDO
               

              ! Update mass (density and thickness), heat and grain size:
              ! ------------------------------------------------------------
              !
              snowrho_pft(ji,:,jv)    = zsnowrhon_pft(ji,:,jv)
              snowheat_pft(ji,:,jv)   = zsnowheatn_pft(ji,:,jv)
              snowgrain_pft(ji,:,jv)  = zsnowgrainn_pft(ji,:,jv)

           ENDIF

        ENDDO
     ENDDO
     !yx:-----

  END SUBROUTINE explicitsnow_transf

  
!!
!================================================================================================================================
!! SU
!!
!>\BRIEF    Computes snowfall    
!!                
!! DESCRIPTION  : Computes snowfall    
!routine. 
!! RECENT CHANGE(S) : None 
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : 
!!
!! FLOWCHART    : None
!! \n
!_
!================================================================================================================================
 
   SUBROUTINE explicitsnow_fall(kjpindex,precip_snow,temp_air,u,v,totfrac_nobio,snowrho,snowdz,&
                        & snowheat,snowgrain,snowtemp,psnowhmass,&
                        & snowtemp_pft, psnowhmass_pft, snowrho_pft, snowdz_pft, snowheat_pft, snowgrain_pft) !yx:+-

       !! 0.1 Input variables

       INTEGER(i_std),INTENT(in)                              :: kjpindex            !! Domain size
       REAL(r_std),DIMENSION(kjpindex),INTENT(in)             :: precip_snow         !! Snow rate (SWE) (kg/m2 per dt_sechiba)
       REAL(r_std),DIMENSION(kjpindex),INTENT(in)             :: temp_air            !! Air temperature
       REAL(r_std),DIMENSION(kjpindex),INTENT(in)             :: u,v                 !! Horizontal wind speed
       REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(in)       :: snowtemp            !! Snow temperature
       REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(in)   :: snowtemp_pft        !! Snow temperature !yx:+-
       REAL(r_std),DIMENSION(kjpindex),INTENT(in)             :: totfrac_nobio
       !! 0.2 Output variables

       REAL(r_std), DIMENSION(kjpindex),INTENT(out)           :: psnowhmass          !! Heat content of snowfall (J/m2)
       REAL(r_std), DIMENSION(kjpindex,nvm),INTENT(out)       :: psnowhmass_pft      !! Heat content of snowfall (J/m2) !yx:+-

       !! 0.3 Modified variables
 
       REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(inout)    :: snowrho             !! Snow density profile (kg/m3)
       REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(inout):: snowrho_pft         !! Snow density profile (kg/m3) !yx:+-
       REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(inout)    :: snowdz              !! Snow layer thickness profile (m)
       REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(inout):: snowdz_pft          !! Snow layer thickness profile (m) !yx:+-
       REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(inout)    :: snowheat            !! Snow heat content/enthalpy (J/m2)
       REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(inout):: snowheat_pft        !! Snow heat content/enthalpy (J/m2) !yx:+-
       REAL(r_std),DIMENSION(kjpindex,nsnow),INTENT(inout)    :: snowgrain           !! Snow grain size (m)
       REAL(r_std),DIMENSION(kjpindex,nsnow,nvm),INTENT(inout):: snowgrain_pft       !! Snow grain size (m) !yx:+-

       !! 0.4 Local variables

       REAL(r_std), DIMENSION(kjpindex)                       :: rhosnew             !! Snowfall density
       REAL(r_std), DIMENSION(kjpindex)                       :: dsnowfall           !! Snowfall thickness (m)
       REAL(r_std), DIMENSION(kjpindex,nsnow)                 :: snowdz_old
       REAL(r_std), DIMENSION(kjpindex,nsnow,nvm)             :: snowdz_old_pft !yx:+-
       REAL(r_std), DIMENSION(kjpindex)                       :: snow_depth,snow_depth_old,newgrain
       REAL(r_std), DIMENSION(kjpindex,nvm)                   :: snow_depth_pft,snow_depth_old_pft !yx:+-
       REAL(r_std)                                            :: snowfall_delta,speed
       INTEGER(i_std)                                         :: ji,jj,jv !yx:+-

       !! 1. initialize the variables

       snowdz_old = snowdz
       DO ji=1,kjpindex
          snow_depth(ji) = SUM(snowdz(ji,:))
       ENDDO

       snow_depth_old = snow_depth

       !yx:+++++
       snowdz_old_pft = snowdz_pft
       DO ji=1,kjpindex
          DO jv=1,nvm
             IF ( ok_LAIdev_sechiba(jv) ) THEN
                 snow_depth_pft(ji,jv) = SUM(snowdz_pft(ji,:,jv))
             ELSE
                 snow_depth_pft(ji,jv) = SUM(snowdz(ji,:))
             ENDIF
          ENDDO
       ENDDO

       snow_depth_old_pft = snow_depth_pft
       !yx:-----
 
       snowfall_delta = 0.0 

       !! 2. incorporate snowfall into snowpack 
       DO ji = 1, kjpindex
   
          speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))
        
          ! new snow fall on snowpack 
          ! NOTE: when the surface temperature is zero, it means that snowfall has no
          ! heat content but it can be used for increasing the thickness and changing the density (maybe it is a bug)
          psnowhmass(ji) = 0.0
          psnowhmass_pft(ji,:) = 0.0 !yx:+-

          IF ( (precip_snow(ji) .GT. 0.0) ) THEN

             !calculate

             psnowhmass(ji) = precip_snow(ji)*(un-totfrac_nobio(ji))* &
                                (xci*(snowtemp(ji,1)-tp_00)-chalfu0)
             !yx:+++++
             DO jv=1,nvm
                IF ( ok_LAIdev_sechiba(jv) ) THEN
                    psnowhmass_pft(ji,jv) = precip_snow(ji)*(un-totfrac_nobio(ji))* &
                                   (xci*(snowtemp_pft(ji,1,jv)-tp_00)-chalfu0)
                ELSE
                   psnowhmass_pft(ji,jv) = precip_snow(ji)*(un-totfrac_nobio(ji))* &
                                   (xci*(snowtemp(ji,1)-tp_00)-chalfu0)
                ENDIF
             ENDDO
             !yx:-----
            
             ! Snowfall density: Following CROCUS (Pahaut 1976)
             !
             rhosnew(ji)   = MAX(xrhosmin, snowfall_a_sn + snowfall_b_sn*(temp_air(ji)-tp_00)+         &
                   snowfall_c_sn*SQRT(speed))

             ! Augment total pack depth:
             ! Note that dsnowfall(ji) can be equal to zero if totfrac_nobio(ji)=1
             dsnowfall(ji) = (precip_snow(ji)*(un-totfrac_nobio(ji)))/rhosnew(ji) !snowfall thickness (m)

             snow_depth(ji) = snow_depth(ji) + dsnowfall(ji) 
             !yx:+++++
             DO jv=1,nvm
                snow_depth_pft(ji,jv) = snow_depth_pft(ji,jv) + dsnowfall(ji)
             ENDDO
             !yx:-----

             ! Fresh snowfall changes the snowpack density and liquid content in uppermost layer 

             IF (dsnowfall(ji) .GT. zero) THEN
               snowrho(ji,1) = (snowdz(ji,1)*snowrho(ji,1) + dsnowfall(ji)*rhosnew(ji))/     &
                                (snowdz(ji,1)+dsnowfall(ji))
               !yx:+++++
               DO jv=1,nvm
                 snowrho_pft(ji,1,jv) = (snowdz_pft(ji,1,jv)*snowrho_pft(ji,1,jv) + dsnowfall(ji)*rhosnew(ji))/     &
                                  (snowdz_pft(ji,1,jv)+dsnowfall(ji))
               ENDDO
               !yx:-----

               snowdz(ji,1) = snowdz(ji,1) + dsnowfall(ji)
               !yx:+++++
               DO jv=1,nvm
                 snowdz_pft(ji,1,jv) = snowdz_pft(ji,1,jv) + dsnowfall(ji)
               ENDDO
               !yx:-----

               ! Add energy of snowfall to snowpack:
               ! Update heat content (J/m2) (therefore the snow temperature
               ! and liquid content):
               !
               snowheat(ji,1)  = snowheat(ji,1) + psnowhmass(ji)
               !yx:+++++
               DO jv=1,nvm
                  snowheat_pft(ji,1,jv)  = snowheat_pft(ji,1,jv) + psnowhmass_pft(ji,jv)
               ENDDO
               !yx:-----
               !
               ! Incorporate snowfall grain size:
               !
               newgrain(ji)    = MIN(dgrain_new_max, snow3lgrain_0d(rhosnew(ji)))
                IF (writeforcheck_explicitsnow) THEN
                  WRITE(numout,*) 'in explicitsnow_fall, before incorporate'
                  WRITE(numout,*) 'yi debug: snowgrain(1,1)=', snowgrain(1,1)
                  WRITE(numout,*) 'yi debug: snowgrain_pft(1,1,:)=', snowgrain_pft(1,1,:)
               ENDIF              
               snowgrain(ji,1) = (snowdz_old(ji,1)*snowgrain(ji,1) + dsnowfall(ji)*newgrain(ji))/ &
                    snowdz(ji,1)
               !yx:+++++
               DO jv=1,nvm
                  snowgrain_pft(ji,1,jv) = (snowdz_old_pft(ji,1,jv)*snowgrain_pft(ji,1,jv) + dsnowfall(ji)*newgrain(ji))/ &
                       snowdz_pft(ji,1,jv)
               ENDDO

               IF (writeforcheck_explicitsnow) THEN
                  WRITE(numout,*) 'in explicitsnow_fall, after incorporate'
                  WRITE(numout,*) 'yi debug: snowdz_old(1,1)=', snowdz_old(1,1)
                  WRITE(numout,*) 'yi debug: snowdz_old_pft(1,1,:)=', snowdz_old_pft(1,1,:)
                  WRITE(numout,*) 'yi debug: snowgrain(1,1)=', snowgrain(1,1)
                  WRITE(numout,*) 'yi debug: snowgrain_pft(1,1,:)=', snowgrain_pft(1,1,:)
                  WRITE(numout,*) 'yi debug: snowdz(1,1)=', snowdz(1,1)
                  WRITE(numout,*) 'yi debug: snowdz_pft(1,1,:)=', snowdz_pft(1,1,:)
                  WRITE(numout,*) 'yi debug: dsnowfall(1)=', dsnowfall(1)
                  WRITE(numout,*) 'yi debug: newgrain(1)=', newgrain(1)
               ENDIF
               !yx:-----
            ENDIF
          ELSE
            dsnowfall(ji) = 0.
          ENDIF

          ! new snow fall on snow free surface. 
          ! we use the linearization for the new snow fall on snow-free ground 

          IF ( (dsnowfall(ji) .GT. zero) .AND. (snow_depth_old(ji) .EQ. zero) ) THEN

             snowfall_delta = 1.0
              
             DO jj=1,nsnow

                snowdz(ji,jj)   = snowfall_delta*(dsnowfall(ji)/nsnow) + &
                                  (1.0-snowfall_delta)*snowdz(ji,jj)

                snowheat(ji,jj)   = snowfall_delta*(psnowhmass(ji)/nsnow) + &
                                  (1.0-snowfall_delta)*snowheat(ji,jj)

                snowrho(ji,jj)    = snowfall_delta*rhosnew(ji)            + &
                                  (1.0-snowfall_delta)*snowrho(ji,jj)

                snowgrain(ji,jj)=   snowfall_delta*newgrain(ji)           + &
                                  (1.0-snowfall_delta)*snowgrain(ji,jj)

             ENDDO


          ENDIF

          !yx:+++++
          DO jv=1,nvm
             IF ( (dsnowfall(ji) .GT. zero) .AND. (snow_depth_old_pft(ji,jv) .EQ. zero) ) THEN
   
                snowfall_delta = 1.0

                DO jj=1,nsnow

                   snowdz_pft(ji,jj,jv)   = snowfall_delta*(dsnowfall(ji)/nsnow) + &
                                     (1.0-snowfall_delta)*snowdz_pft(ji,jj,jv)

                   snowheat_pft(ji,jj,jv)   = snowfall_delta*(psnowhmass_pft(ji,jv)/nsnow) + &
                                     (1.0-snowfall_delta)*snowheat_pft(ji,jj,jv)

                   snowrho_pft(ji,jj,jv)    = snowfall_delta*rhosnew(ji)            + &
                                     (1.0-snowfall_delta)*snowrho_pft(ji,jj,jv)

                   snowgrain_pft(ji,jj,jv)=   snowfall_delta*newgrain(ji)           + &
                                     (1.0-snowfall_delta)*snowgrain_pft(ji,jj,jv)

                ENDDO

             ENDIF
          ENDDO
          !yx:-----

       ENDDO 

     END SUBROUTINE explicitsnow_fall

!!
!================================================================================================================================
!! SUBROUTINE   : explicitsnow_gone
!!
!>\BRIEF        Check whether snow is gone 
!!                
!! DESCRIPTION  : If so, set thickness (and therefore mass and heat) and liquid
!!                content to zero, and adjust fluxes of water, evaporation and
!!                heat into underlying surface. 
!! RECENT CHANGE(S) : None 
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : 
!!
!! FLOWCHART    : None
!! \n
!_
!================================================================================================================================

   SUBROUTINE explicitsnow_gone(kjpindex,pgflux,&
                        snowheat,snowtemp,snowdz,snowrho,snowliq,grndflux,snowmelt,soilflxresid, &
                        pgflux_pft, snowheat_pft, soilflxresid_pft, snowtemp_pft, snowdz_pft, snowrho_pft, & !yx:+-
                        snowliq_pft, grndflux_pft, snowmelt_pft, thrufal_pft) !yx:+-

     !! 0.1 Input variables

     INTEGER(i_std), INTENT(in)                                 :: kjpindex     !! Domain size
     REAL(r_std),DIMENSION (kjpindex), INTENT (in)              :: pgflux       !! Net energy into snow pack(w/m2)
     REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)          :: pgflux_pft   !! Net energy into snow pack(w/m2) !yx:+-
     REAL(r_std),DIMENSION (kjpindex,nsnow),INTENT(in)          :: snowheat     !! Snow heat content (J/m^2)
     REAL(r_std),DIMENSION (kjpindex,nsnow,nvm),INTENT(in)          :: snowheat_pft !! Snow heat content (J/m^2) !yx:+-
     REAL(r_std),DIMENSION(kjpindex),INTENT(in)                 :: soilflxresid
     REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)          :: soilflxresid_pft !yx:+-

     !! 0.2 Output variables

     !! 0.3 Modified variables

     REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(inout)     :: snowtemp     !! Snow temperature
     REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(inout) :: snowtemp_pft !! Snow temperature !yx:+-
     REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(inout)     :: snowdz       !! Snow depth
     REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(inout) :: snowdz_pft   !! Snow depth !yx:+-
     REAL(r_std),DIMENSION (kjpindex,nsnow), INTENT(inout)      :: snowrho      !! Snow density (Kg/m^3)
     REAL(r_std),DIMENSION (kjpindex,nsnow,nvm), INTENT(inout)  :: snowrho_pft  !! Snow density (Kg/m^3) !yx:+-
     REAL(r_std),DIMENSION (kjpindex,nsnow), INTENT(inout)      :: snowliq      !! Liquid water content
     REAL(r_std),DIMENSION (kjpindex,nsnow,nvm), INTENT(inout)  :: snowliq_pft  !! Liquid water content !yx:+-
     REAL(r_std),DIMENSION(kjpindex), INTENT(inout)             :: grndflux     !! Soil/snow interface heat flux (W/m2)
     REAL(r_std),DIMENSION(kjpindex,nvm), INTENT(inout)         :: grndflux_pft !! Soil/snow interface heat flux (W/m2) !yx:+-
     REAL(r_std),DIMENSION(kjpindex),INTENT(inout)              :: snowmelt     !! Snow melt
     REAL(r_std),DIMENSION(kjpindex,nvm), INTENT(inout)         :: snowmelt_pft !! Snow melt !yx:+-
     REAL(r_std),DIMENSION(kjpindex)                            :: thrufal      !! Water leaving snowpack(kg/m2/s)
     REAL(r_std),DIMENSION(kjpindex,nvm)                        :: thrufal_pft  !! Water leaving snowpack(kg/m2/s) !yx:+-

     !! 0.4 Local variables

     INTEGER(i_std)                                             :: ji,jj,jv !yx:+-
     REAL(r_std),DIMENSION(kjpindex)                            :: snowgone_delta
     REAL(r_std),DIMENSION(kjpindex,nvm)                        :: snowgone_delta_pft !yx:+-
     REAL(r_std),DIMENSION (kjpindex)                           :: totsnowheat  !!snow heat content at each layer 
     REAL(r_std),DIMENSION(kjpindex,nvm)                        :: totsnowheat_pft !!snow heat content at each layer !yx:+-
     REAL(r_std),DIMENSION(kjpindex)                            :: snowdepth_crit
     REAL(r_std),DIMENSION(kjpindex,nvm)                        :: snowdepth_crit_pft !yx:+-

     ! first caculate total snowpack snow heat content
     snowgone_delta(:) = un
     thrufal(:)=0.0
     snowmelt(:)=0
     totsnowheat(:)  = SUM(snowheat(:,:),2) 

     !yx:+++++
     snowgone_delta_pft(:,:) = un
     thrufal_pft(:,:)=0.0
     snowmelt_pft(:,:)=0
     totsnowheat_pft(:,:)  = SUM(snowheat_pft(:,:,:),2)
     !yx:-----
     
     DO ji = 1, kjpindex 

           IF ( (SUM(snowdz(ji,:)) .GT. min_sechiba)) THEN

              IF( pgflux(ji) + soilflxresid(ji) >= (-totsnowheat(ji)/dt_sechiba) ) THEN

                 grndflux(ji) = pgflux(ji) + (totsnowheat(ji)/dt_sechiba) + soilflxresid(ji) 

                 thrufal(ji)=SUM(snowrho(ji,:)*snowdz(ji,:))

                 snowgone_delta(ji) = 0.0       

                 snowmelt(ji) = snowmelt(ji)+thrufal(ji)

              ENDIF

              ! update of snow state (either still present or not)

              DO jj=1,nsnow
                 snowdz(ji,jj)  =   snowdz(ji,jj) *snowgone_delta(ji)
                 snowliq(ji,jj)   =   snowliq(ji,jj) *snowgone_delta(ji)
                 snowtemp(ji,jj) = (1.0-snowgone_delta(ji))*tp_00 + snowtemp(ji,jj)*snowgone_delta(ji)
              ENDDO

           ELSE
	      snowdz(ji,:)=0.
              snowliq(ji,:)=0.
	      snowmelt(ji)=snowmelt(ji)+SUM(snowrho(ji,:)*snowdz(ji,:))
           ENDIF
     ENDDO 


     !yx:+++++
     DO ji = 1, kjpindex
        DO jv=1, nvm

           IF ( (SUM(snowdz_pft(ji,:,jv)) .GT. min_sechiba)) THEN

              IF( pgflux_pft(ji,jv) + soilflxresid_pft(ji,jv) >= (-totsnowheat_pft(ji,jv)/dt_sechiba) ) THEN

                 grndflux_pft(ji,jv) = pgflux_pft(ji,jv) + (totsnowheat_pft(ji,jv)/dt_sechiba) + soilflxresid_pft(ji,jv)

                 thrufal_pft(ji,jv)=SUM(snowrho_pft(ji,:,jv)*snowdz_pft(ji,:,jv))

                 snowgone_delta_pft(ji,jv) = 0.0

                 snowmelt_pft(ji,jv) = snowmelt_pft(ji,jv)+thrufal_pft(ji,jv)

              ENDIF

              ! update of snow state (either still present or not)

              DO jj=1,nsnow
                 snowdz_pft(ji,jj,jv)  =   snowdz_pft(ji,jj,jv) *snowgone_delta_pft(ji,jv)
                 snowliq_pft(ji,jj,jv)   =   snowliq_pft(ji,jj,jv) *snowgone_delta_pft(ji,jv)
                 snowtemp_pft(ji,jj,jv) = (1.0-snowgone_delta_pft(ji,jv))*tp_00 + snowtemp_pft(ji,jj,jv)*snowgone_delta_pft(ji,jv)
              ENDDO

           ELSE
              snowdz_pft(ji,:,jv)=0.
              snowliq_pft(ji,:,jv)=0.
              snowmelt_pft(ji,jv)=snowmelt_pft(ji,jv)+SUM(snowrho_pft(ji,:,jv)*snowdz_pft(ji,:,jv))
           ENDIF

        ENDDO
     ENDDO
     !yx:-----

   END SUBROUTINE explicitsnow_gone

!================================================================================================================================
!! SUBROUTINE   : explicitsnow_melt_refrz
!!
!>\BRIEF        Computes snow melt and refreezing processes within snowpack
!!                
!! DESCRIPTION  : 
!! RECENT CHANGE(S) : None 
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : 
!!
!! FLOWCHART    : None
!! \n
!_
!================================================================================================================================

   SUBROUTINE explicitsnow_melt_refrz(kjpindex,precip_rain,pgflux,soilcap,&
                     snowtemp,snowdz,snowrho,snowliq,snowmelt,grndflux,temp_air,soilflxresid,&
                     pcapa_snow_pft, pgflux_pft, soilcap_pft, soilflxresid_pft, & !yx:+-
                     snowtemp_pft, snowdz_pft, snowrho_pft, snowliq_pft, grndflux_pft, snowmelt_pft) !yx:+-

      !! 0.1 Input variables

      INTEGER(i_std), INTENT (in)                             :: kjpindex     !! Domain size
      REAL(r_std),DIMENSION (kjpindex,nsnow)                  :: pcapa_snow   !! Heat capacity for snow
      REAL(r_std),DIMENSION (kjpindex,nsnow,nvm)                  :: pcapa_snow_pft   !! Heat capacity for snow !yx:+-
      REAL(r_std),DIMENSION (kjpindex), INTENT(in)            :: precip_rain  !! Rainfall      
      REAL(r_std),DIMENSION (kjpindex), INTENT(in)            :: temp_air     !! Air temperature
      REAL(r_std),DIMENSION (kjpindex),INTENT(in)             :: pgflux       !! Net energy into snowpack(w/m2)
      REAL(r_std),DIMENSION (kjpindex,nvm),INTENT(in)         :: pgflux_pft   !! Net energy into snowpack(w/m2) !yx:+-
      REAL(r_std),DIMENSION (kjpindex),INTENT(in)             :: soilcap      !! Soil heat capacity
      REAL(r_std),DIMENSION (kjpindex,nvm),INTENT(in)         :: soilcap_pft  !! Soil heat capacity !yx:+-
      REAL(r_std),DIMENSION (kjpindex),INTENT(in)             :: soilflxresid
      REAL(r_std),DIMENSION (kjpindex,nvm),INTENT(in)         :: soilflxresid_pft !yx:+-

      !! 0.2 Output variables

      !! 0.3 Modified variables

      REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(inout)  :: snowtemp     !! Snow temperature 
      REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(inout)  :: snowtemp_pft !! Snow temperature !yx:+-
      REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(inout)  :: snowdz       !! Snow depth
      REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(inout)  :: snowdz_pft   !! Snow depth !yx:+-
      REAL(r_std),DIMENSION (kjpindex,nsnow), INTENT(inout)   :: snowrho      !! Snow layer density (Kg/m^3)
      REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(inout)  :: snowrho_pft   !! Snow layer density (Kg/m^3) !yx:+-
      REAL(r_std),DIMENSION (kjpindex,nsnow), INTENT(inout)   :: snowliq      !! Liquid water content
      REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(inout)  :: snowliq_pft   !! Liquid water content !yx:+-
      REAL(r_std),DIMENSION(kjpindex),INTENT(inout)           :: grndflux     !! Net energy input to soil 
      REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout)       :: grndflux_pft     !! Net energy input to soil !yx:+-
      REAL(r_std),DIMENSION (kjpindex), INTENT(inout)         :: snowmelt     !! Snowmelt
      REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout)       :: snowmelt_pft     !! Snowmelt !yx:+-

      !! 0.4 Local variables

      !yx:++++++ 
      !add var_pft for PFT-specific processes
      REAL(r_std),DIMENSION (kjpindex)                        :: meltxs       !! Residual snowmelt energy applied to underlying soil
      REAL(r_std),DIMENSION (kjpindex,nvm)                    :: meltxs_pft   !! Residual snowmelt energy applied to underlying soil !yx:+-
      !REAL(r_std)                                             :: enerin,melttot,hrain !yx:+- commented, unused 
      REAL(r_std),DIMENSION (nsnow)                           :: zsnowlwe, zsnowlwe_pft
      REAL(r_std),DIMENSION (nsnow)                           :: flowliq, flowliq_pft
      REAL(r_std),DIMENSION (kjpindex)                        :: snowmass
      REAL(r_std),DIMENSION (kjpindex,nvm)                    :: snowmass_pft !yx:+-
      REAL(r_std),DIMENSION (nsnow)                           :: zphase, zphase_pft       !! Phase change (from ice to water) (J/m2)
      REAL(r_std),DIMENSION (nsnow)                           :: zphase2, zphase2_pft      !! Phase change (from water to ice)
      !REAL(r_std),DIMENSION (nsnow)                           :: zphase3      !! Phase change related with net energy input to snowpack !yx:+- commented, unused
      REAL(r_std),DIMENSION (nsnow)                           :: zsnowdz, zsnowdz_pft      !! Snow layer depth
      REAL(r_std),DIMENSION (nsnow)                           :: zsnowmelt, zsnowmelt_pft    !! Snow melt (liquid water) (m)
      REAL(r_std),DIMENSION (nsnow)                           :: zsnowtemp, zsnowtemp_pft
      REAL(r_std),DIMENSION (nsnow)                           :: zmeltxs, zmeltxs_pft      !! Excess melt
      REAL(r_std),DIMENSION (nsnow)                           :: zwholdmax, zwholdmax_pft    !! Maximum liquid water holding (m)
      REAL(r_std),DIMENSION (nsnow)                           :: zcmprsfact, zcmprsfact_pft   !! Compression factor due to densification from melting
      REAL(r_std),DIMENSION (nsnow)                           :: zscap, zscap_pft        !! Snow heat capacity (J/m3 K)
      REAL(r_std),DIMENSION (nsnow)                           :: zsnowliq, zsnowliq_pft     !! (m)
      REAL(r_std),DIMENSION (nsnow)                           :: snowtemp_old, snowtemp_old_pft
      REAL(r_std),DIMENSION (0:nsnow)                         :: zflowliqt, zflowliqt_pft    !!(m)
      REAL(r_std)                                             :: zrainfall,zpcpxs, zrainfall_pft, zpcpxs_pft
      REAL(r_std)                                             :: ztotwcap, ztotwcap_pft
      !REAL(r_std),DIMENSION(kjpindex,nsnow)                   :: snowdz_old,snowliq_old !yx:+- commented, unused
      INTEGER(i_std)                                          :: jj,ji, iv,jv
      !REAL(r_std),DIMENSION(nsnow)                            :: snowdz_old2 !yx:+- uncommented
      !REAL(r_std),DIMENSION(nsnow)                            :: zsnowrho
      !yx:-----
    
      !initialize
      !snowdz_old = snowdz !yx:+- commented
      !snowliq_old = snowliq !yx:+- commented

      DO ji = 1, kjpindex


         snowmass(ji) = SUM(snowrho(ji,:) * snowdz(ji,:))
         IF ((snowmass(ji) .GT. min_sechiba)) THEN

           !! 1 snow melting due to positive snowpack snow temperature
 
             !! 1.0 total liquid equivalent water content of each snow layer

               zsnowlwe(:) = snowrho(ji,:) * snowdz(ji,:)/ ph2o 

             !! 1.1 phase change (J/m2)

               pcapa_snow(ji,:) = snowrho(ji,:)*xci

               !we should make a change in order to avoid the remaining energy
               !still be left without melting the underlying two snow layers

               zphase(:)  = MIN(pcapa_snow(ji,:)*MAX(0.0, snowtemp(ji,:)-tp_00)*      &
                                snowdz(ji,:),                                       &
                            MAX(0.0,zsnowlwe(:)-snowliq(ji,:))*chalfu0*ph2o)

             !! 1.2 update snow liq water and temperature if melting

               zsnowmelt(:) = zphase(:)/(chalfu0*ph2o)

             !! 1.3 cool off the snow layer temperature due to melt

               zsnowtemp(:) = snowtemp(ji,:) - zphase(:)/(pcapa_snow(ji,:)* snowdz(ji,:))

               snowtemp(ji,:) = MIN(tp_00, zsnowtemp(:))

               zmeltxs(:)   = (zsnowtemp(:)-snowtemp(ji,:))*pcapa_snow(ji,:)*snowdz(ji,:)

             !! 1.4 loss of snowpack depth and liquid equivalent water

               zwholdmax(:) = snow3lhold_1d(snowrho(ji,:),snowdz(ji,:)) ! 1 dimension

               zcmprsfact(:) = (zsnowlwe(:)-MIN(snowliq(ji,:)+zsnowmelt(:),zwholdmax(:)))/ &
                               (zsnowlwe(:)-MIN(snowliq(ji,:),zwholdmax(:)))

               snowdz(ji,:)    = snowdz(ji,:)*zcmprsfact(:)

               snowrho(ji,:)     = zsnowlwe(:)*ph2o/snowdz(ji,:)

               snowliq(ji,:)   = snowliq(ji,:) + zsnowmelt(:)


           !! 2 snow refreezing process
              !! 2.1 freeze liquid water in any layer
               zscap(:)     = snowrho(ji,:)*xci  !J K-1 m-3
               zphase2(:)    = MIN(zscap(:)*                                          &
                                MAX(0.0, tp_00 - snowtemp(ji,:))*snowdz(ji,:),             &
                                snowliq(ji,:)*chalfu0*ph2o)

               ! warm layer and reduce liquid if freezing occurs
               zsnowdz(:) = MAX(xsnowdmin/nsnow, snowdz(ji,:))
               snowtemp_old(:) = snowtemp(ji,:) 
               snowtemp(ji,:) = snowtemp(ji,:) + zphase2(:)/(zscap(:)*zsnowdz(:))
               
               ! Reduce liquid portion if freezing occurs:
               snowliq(ji,:) = snowliq(ji,:) - ( (snowtemp(ji,:)-snowtemp_old(:))*       &
               zscap(:)*zsnowdz(:)/(chalfu0*ph2o) )
               snowliq(ji,:) = MAX(snowliq(ji,:), 0.0)
           !! 3. thickness change due to snowmelt in excess of holding capacity
               zwholdmax(:) = snow3lhold_1d(snowrho(ji,:),snowdz(ji,:)) ! 1 dimension
               flowliq(:) = MAX(0.,(snowliq(ji,:)-zwholdmax(:)))
               snowliq(ji,:)  = snowliq(ji,:) - flowliq(:)
               snowdz(ji,:) = snowdz(ji,:) - flowliq(:)*ph2o/snowrho(ji,:)
               ! to prevent possible very small negative values (machine
               ! prescision as snow vanishes
               snowdz(ji,:) = MAX(0.0, snowdz(ji,:)) 

           !! 4. liquid water flow
               ztotwcap = SUM(zwholdmax(:)) 
               ! Rain entering snow (m):
               !ORCHIDEE has assumed that all precipitation entering snow has
               !left the snowpack and finally become runoff in hydrolc_soil. Here we just put zrainfall as 0.0
               !!zrainfall = precip_rain(ji)/ph2o ! rainfall (m)
               zrainfall = 0.0

               zflowliqt(0) = MIN(zrainfall,ztotwcap)
               ! Rain assumed to directly pass through the pack to runoff (m):
               zpcpxs = zrainfall - zflowliqt(0)
               !
               DO jj=1,nsnow
                  zflowliqt(jj) = flowliq(jj)
               ENDDO

               ! translated into a density increase:
               flowliq(:) = 0.0                ! clear this array for work
               zsnowliq(:) = snowliq(ji,:)      ! reset liquid water content
               !

               DO jj=1,nsnow
                  snowliq(ji,jj)  = snowliq(ji,jj) + zflowliqt(jj-1)
                  flowliq(jj)   = MAX(0.0, (snowliq(ji,jj)-zwholdmax(jj)))
                  snowliq(ji,jj)  = snowliq(ji,jj) - flowliq(jj)

                  !Modified by Tao Wang based on previous ISBA-ES scheme
                  snowrho(ji,jj)  = snowrho(ji,jj)  + (snowliq(ji,jj) - zsnowliq(jj))*       &
                                        & ph2o/MAX(xsnowdmin/nsnow,snowdz(ji,jj))

                  zflowliqt(jj) = zflowliqt(jj) + flowliq(jj)  
               ENDDO

               snowmelt(ji)  = snowmelt(ji) + (zflowliqt(nsnow) + zpcpxs) *  ph2o

             ! excess heat from melting, using it to warm underlying ground to conserve energy
               meltxs(ji) = SUM(zmeltxs(:))/dt_sechiba  ! (W/m2)

             ! energy flux into the soil 
               grndflux(ji) = grndflux(ji) + meltxs(ji)  

         ELSE
	       snowdz(ji,:)=0.
               snowliq(ji,:)=0.
	       snowmelt(ji)=snowmelt(ji)+SUM(snowrho(ji,:)*snowdz(ji,:))
         ENDIF

      ENDDO


      !yx:+++++
      DO ji = 1, kjpindex
         DO jv=1,nvm

            snowmass_pft(ji,jv) = SUM(snowrho_pft(ji,:,jv) * snowdz_pft(ji,:,jv))
            IF ((snowmass_pft(ji,jv) .GT. min_sechiba)) THEN

              !! 1 snow melting due to positive snowpack snow temperature

                !! 1.0 total liquid equivalent water content of each snow layer

                  zsnowlwe_pft(:) = snowrho_pft(ji,:,jv) * snowdz_pft(ji,:,jv)/ ph2o

                !! 1.1 phase change (J/m2)

                  pcapa_snow_pft(ji,:,jv) = snowrho_pft(ji,:,jv)*xci

                  !we should make a change in order to avoid the remaining energy
                  !still be left without melting the underlying two snow layers

                  zphase_pft(:)  = MIN(pcapa_snow_pft(ji,:,jv)*MAX(0.0, snowtemp_pft(ji,:,jv)-tp_00)*      &
                                   snowdz_pft(ji,:,jv),                                       &
                               MAX(0.0,zsnowlwe_pft(:)-snowliq_pft(ji,:,jv))*chalfu0*ph2o)

                !! 1.2 update snow liq water and temperature if melting

                  zsnowmelt_pft(:) = zphase_pft(:)/(chalfu0*ph2o)

                !! 1.3 cool off the snow layer temperature due to melt

                  zsnowtemp_pft(:) = snowtemp_pft(ji,:,jv) - zphase_pft(:)/(pcapa_snow_pft(ji,:,jv)* snowdz_pft(ji,:,jv))

                  snowtemp_pft(ji,:,jv) = MIN(tp_00, zsnowtemp_pft(:))

                  zmeltxs_pft(:)   = (zsnowtemp_pft(:)-snowtemp_pft(ji,:,jv))*pcapa_snow_pft(ji,:,jv)*snowdz_pft(ji,:,jv)

                !! 1.4 loss of snowpack depth and liquid equivalent water

                  zwholdmax_pft(:) = snow3lhold_1d(snowrho_pft(ji,:,jv),snowdz_pft(ji,:,jv)) ! 1 dimension

                  zcmprsfact_pft(:) = (zsnowlwe_pft(:)-MIN(snowliq_pft(ji,:,jv)+zsnowmelt_pft(:),zwholdmax_pft(:)))/ &
                                  (zsnowlwe_pft(:)-MIN(snowliq_pft(ji,:,jv),zwholdmax_pft(:)))

                  snowdz_pft(ji,:,jv)    = snowdz_pft(ji,:,jv)*zcmprsfact_pft(:)

                  snowrho_pft(ji,:,jv)     = zsnowlwe_pft(:)*ph2o/snowdz_pft(ji,:,jv)

                  snowliq_pft(ji,:,jv)   = snowliq_pft(ji,:,jv) + zsnowmelt_pft(:)

              !! 2 snow refreezing process
                 !! 2.1 freeze liquid water in any layer
                  zscap_pft(:)     = snowrho_pft(ji,:,jv)*xci  !J K-1 m-3
                  zphase2_pft(:)    = MIN(zscap_pft(:)*                                          &
                                   MAX(0.0, tp_00 - snowtemp_pft(ji,:,jv))*snowdz_pft(ji,:,jv),             &
                                   snowliq_pft(ji,:,jv)*chalfu0*ph2o)

                  ! warm layer and reduce liquid if freezing occurs
                  zsnowdz_pft(:) = MAX(xsnowdmin/nsnow, snowdz_pft(ji,:,jv))
                  snowtemp_old_pft(:) = snowtemp_pft(ji,:,jv)
                  snowtemp_pft(ji,:,jv) = snowtemp_pft(ji,:,jv) + zphase2_pft(:)/(zscap_pft(:)*zsnowdz_pft(:))

                  ! Reduce liquid portion if freezing occurs:
                  snowliq_pft(ji,:,jv) = snowliq_pft(ji,:,jv) - ( (snowtemp_pft(ji,:,jv)-snowtemp_old_pft(:))*       &
                  zscap_pft(:)*zsnowdz_pft(:)/(chalfu0*ph2o) )
                  snowliq_pft(ji,:,jv) = MAX(snowliq_pft(ji,:,jv), 0.0)

              !! 3. thickness change due to snowmelt in excess of holding capacity
                  zwholdmax_pft(:) = snow3lhold_1d(snowrho_pft(ji,:,jv),snowdz_pft(ji,:,jv)) ! 1 dimension
                  flowliq_pft(:) = MAX(0.,(snowliq_pft(ji,:,jv)-zwholdmax_pft(:)))
                  snowliq_pft(ji,:,jv)  = snowliq_pft(ji,:,jv) - flowliq_pft(:)
                  snowdz_pft(ji,:,jv) = snowdz_pft(ji,:,jv) - flowliq_pft(:)*ph2o/snowrho_pft(ji,:,jv)
                  ! to prevent possible very small negative values (machine
                  ! prescision as snow vanishes
                  snowdz_pft(ji,:,jv) = MAX(0.0, snowdz_pft(ji,:,jv))

              !! 4. liquid water flow
                  ztotwcap_pft = SUM(zwholdmax_pft(:))
                  ! Rain entering snow (m):
                  !ORCHIDEE has assumed that all precipitation entering snow has
                  !left the snowpack and finally become runoff in hydrolc_soil. Here we just put zrainfall as 0.0
                  !!zrainfall = precip_rain(ji)/ph2o ! rainfall (m)
                  zrainfall_pft = 0.0

                  zflowliqt_pft(0) = MIN(zrainfall_pft,ztotwcap_pft)
                  ! Rain assumed to directly pass through the pack to runoff (m):
                  zpcpxs_pft = zrainfall_pft - zflowliqt_pft(0)
                  !
                  DO jj=1,nsnow
                     zflowliqt_pft(jj) = flowliq_pft(jj)
                  ENDDO

                  ! translated into a density increase:
                  flowliq_pft(:) = 0.0                ! clear this array for work
                  zsnowliq_pft(:) = snowliq_pft(ji,:,jv)      ! reset liquid water content
                  !

                  DO jj=1,nsnow
                     snowliq_pft(ji,jj,jv)  = snowliq_pft(ji,jj,jv) + zflowliqt_pft(jj-1)
                     flowliq_pft(jj)   = MAX(0.0, (snowliq_pft(ji,jj,jv)-zwholdmax_pft(jj)))
                     snowliq_pft(ji,jj,jv)  = snowliq_pft(ji,jj,jv) - flowliq_pft(jj)

                     !Modified by Tao Wang based on previous ISBA-ES scheme
                     snowrho_pft(ji,jj,jv)  = snowrho_pft(ji,jj,jv)  + (snowliq_pft(ji,jj,jv) - zsnowliq_pft(jj))*       &
                                           & ph2o/MAX(xsnowdmin/nsnow,snowdz_pft(ji,jj,jv))
 
                     zflowliqt_pft(jj) = zflowliqt_pft(jj) + flowliq_pft(jj)
                  ENDDO
  
               snowmelt_pft(ji,jv)  = snowmelt_pft(ji,jv) + (zflowliqt_pft(nsnow) + zpcpxs_pft) *  ph2o

             ! excess heat from melting, using it to warm underlying ground to conserve energy
               meltxs_pft(ji,jv) = SUM(zmeltxs_pft(:))/dt_sechiba  ! (W/m2)

             ! energy flux into the soil
               grndflux_pft(ji,jv) = grndflux_pft(ji,jv) + meltxs_pft(ji,jv)

         ELSE
               snowdz_pft(ji,:,jv)=0.
               snowliq_pft(ji,:,jv)=0.
               snowmelt_pft(ji,jv)=snowmelt_pft(ji,jv)+SUM(snowrho_pft(ji,:,jv)*snowdz_pft(ji,:,jv))
         ENDIF

         ENDDO
      ENDDO
      !yx:-----


    END SUBROUTINE explicitsnow_melt_refrz

!================================================================================================================================
!! SUBROUTINE   : explicitsnow_levels
!!
!>\BRIEF        Computes snow discretization based on given total snow depth
!!                
!! DESCRIPTION  : 
!! RECENT CHANGE(S) : None 
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : 
!!
!! FLOWCHART    : None
!! \n
!_
!================================================================================================================================ 
  SUBROUTINE explicitsnow_levels( kjpindex,snow_thick, snowdz, &
             snow_thick_pft, snowdz_pft) !yx:+-
    
    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                                     :: kjpindex     !! Domain size
    REAL(r_std),DIMENSION (kjpindex),   INTENT (in)                :: snow_thick   !! Total snow depth
    REAL(r_std),DIMENSION (kjpindex,nvm),   INTENT (in)            :: snow_thick_pft !! Total snow depth !yx:+-

    !! 0.2 Output variables

    !! 0.3 Modified variables

    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(inout)         :: snowdz                           !! Snow depth
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm), INTENT(inout)     :: snowdz_pft                       !! Snow depth !yx:+-

    !! 0.4 Local variables

    INTEGER(i_std)                                                 :: il,it,ji,jv !yx:+-
    INTEGER(i_std)                                                 :: i,j, ix
    REAL(r_std), DIMENSION(kjpindex)                               :: xi, xf
    REAL(r_std), PARAMETER, DIMENSION(3)                           :: ZSGCOEF1  = (/0.25, 0.50, 0.25/) !! Snow grid parameters 
    REAL(r_std), PARAMETER, DIMENSION(2)                           :: ZSGCOEF2  = (/0.05, 0.34/)       !! Snow grid parameters 
    REAL(r_std), PARAMETER                                         :: ZSNOWTRANS = 0.20                !! Minimum total snow depth at which surface layer thickness is constant (m)
    REAL(r_std), PARAMETER                                         :: XSNOWCRITD = 0.03                !! (m)

    DO ji=1,kjpindex
       IF ( snow_thick(ji) .LE. (XSNOWCRITD+0.01)) THEN

        snowdz(ji,1) = MIN(0.01, snow_thick(ji)/nsnow)
        snowdz(ji,3) = MIN(0.01, snow_thick(ji)/nsnow)
        snowdz(ji,2) = snow_thick(ji) - snowdz(ji,1) - snowdz(ji,3)
   
       ENDIF 

       !yx:+++++
       DO jv=1,nvm
          IF ( snow_thick_pft(ji,jv) .LE. (XSNOWCRITD+0.01)) THEN
             snowdz_pft(ji,1,jv) = MIN(0.01, snow_thick_pft(ji,jv)/nsnow)
             snowdz_pft(ji,3,jv) = MIN(0.01, snow_thick_pft(ji,jv)/nsnow)
             snowdz_pft(ji,2,jv) = snow_thick_pft(ji,jv) - snowdz_pft(ji,1,jv) - snowdz_pft(ji,3,jv)
          ENDIF
       ENDDO
       !yx:-----
    ENDDO

    WHERE ( snow_thick(:) .LE. ZSNOWTRANS .AND. &
            snow_thick(:) .GT. (XSNOWCRITD+0.01) )
       !
        snowdz(:,1) = snow_thick(:)*ZSGCOEF1(1)
        snowdz(:,2) = snow_thick(:)*ZSGCOEF1(2) 
        snowdz(:,3) = snow_thick(:)*ZSGCOEF1(3)
       !
    END WHERE

    !yx:+++++
    DO jv=1,nvm
       WHERE ( snow_thick_pft(:,jv) .LE. ZSNOWTRANS .AND. &
               snow_thick_pft(:,jv) .GT. (XSNOWCRITD+0.01) )
          snowdz_pft(:,1,jv) = snow_thick_pft(:,jv)*ZSGCOEF1(1)
          snowdz_pft(:,2,jv) = snow_thick_pft(:,jv)*ZSGCOEF1(2)
          snowdz_pft(:,3,jv) = snow_thick_pft(:,jv)*ZSGCOEF1(3)
       END WHERE
    ENDDO
    !yx:-----

    DO ji = 1,kjpindex
       IF (snow_thick(ji) .GT. ZSNOWTRANS) THEN
          snowdz(ji,1) = ZSGCOEF2(1)
          snowdz(ji,2) = (snow_thick(ji)-ZSGCOEF2(1))*ZSGCOEF2(2) + ZSGCOEF2(1)
          !When using simple finite differences, limit the thickness
          !factor between the top and 2nd layers to at most 10
          snowdz(ji,2)  = MIN(10*ZSGCOEF2(1),  snowdz(ji,2) )
          snowdz(ji,3)  = snow_thick(ji) - snowdz(ji,2) - snowdz(ji,1)
       ENDIF

       !yx:+++++
       DO jv = 1,nvm
          IF (snow_thick_pft(ji,jv) .GT. ZSNOWTRANS) THEN
             snowdz_pft(ji,1,jv) = ZSGCOEF2(1)
             snowdz_pft(ji,2,jv) = (snow_thick_pft(ji,jv)-ZSGCOEF2(1))*ZSGCOEF2(2) + ZSGCOEF2(1)
             !When using simple finite differences, limit the thickness
             !factor between the top and 2nd layers to at most 10
             snowdz_pft(ji,2,jv)  = MIN(10*ZSGCOEF2(1),  snowdz_pft(ji,2,jv) )
             snowdz_pft(ji,3,jv)  = snow_thick_pft(ji,jv) - snowdz_pft(ji,2,jv) - snowdz_pft(ji,1,jv)
          ENDIF
       ENDDO
       !yx:-----
    ENDDO      

  END SUBROUTINE explicitsnow_levels

!!
!================================================================================================================================
!! SUBROUTINE   : explicitsnow_profile
!!
!>\BRIEF        
!!
!! DESCRIPTION  : In this routine solves the numerical soil thermal scheme, ie calculates the new soil temperature profile.
!!
!! RECENT CHANGE(S) : None
!! 
!! MAIN OUTPUT VARIABLE(S): 
!!
!! REFERENCE(S) :
!!
!! FLOWCHART    : None 
!! \n 
!_
!================================================================================================================================
  SUBROUTINE explicitsnow_profile (kjpindex, cgrnd_snow,dgrnd_snow,lambda_snow,temp_sol_new, snowtemp,snowdz,temp_sol_add,&
             temp_sol_new_pft,cgrnd_snow_pft,dgrnd_snow_pft,lambda_snow_pft,snowdz_pft,temp_sol_add_pft,snowtemp_pft)!yx:+-

  !! 0. Variables and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                               :: kjpindex     !! Domain size (unitless)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: temp_sol_new !! skin temperature
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)        :: temp_sol_new_pft !! skin temperature !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow),INTENT (in)      :: cgrnd_snow   !! Integration coefficient for snow numerical scheme
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm),INTENT (in)  :: cgrnd_snow_pft   !! Integration coefficient for snow numerical scheme !yx:+-
    REAL(r_std), DIMENSION (kjpindex,nsnow),INTENT (in)      :: dgrnd_snow   !! Integration coefficient for snow numerical scheme
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm),INTENT (in)  :: dgrnd_snow_pft   !! Integration coefficient for snow numerical scheme !yx:+-
    REAL(r_std), DIMENSION (kjpindex),INTENT(in)             :: lambda_snow  !! Coefficient of the linear extrapolation of surface temperature
    REAL(r_std), DIMENSION (kjpindex,nvm),INTENT(in)         :: lambda_snow_pft  !! Coefficient of the linear extrapolation of surface temperature !yx:+- 
    REAL(r_std), DIMENSION (kjpindex,nsnow),INTENT(in)       :: snowdz       !! Snow layer thickness
    REAL(r_std), DIMENSION (kjpindex,nsnow,nvm),INTENT(in)       :: snowdz_pft       !! Snow layer thickness !yx:+-
    REAL(r_std), DIMENSION (kjpindex),INTENT(inout)          :: temp_sol_add !! Additional energy to melt snow for snow ablation case (K)
    REAL(r_std), DIMENSION (kjpindex,nvm),INTENT(inout)      :: temp_sol_add_pft !! Additional energy to melt snow for snow ablation case (K) !yx:+-

    !! 0.3 Modified variable

    !! 0.2 Output variables

    !! 0.3 Modified variables
    REAL(r_std),DIMENSION (kjpindex,nsnow), INTENT (inout)   :: snowtemp
    REAL(r_std),DIMENSION (kjpindex,nsnow,nvm), INTENT (inout)   :: snowtemp_pft !yx:+-

    !! 0.4 Local variables

    INTEGER(i_std)                                           :: ji, jg, jv !yx:+-
!_
!================================================================================================================================
  !! 1. Computes the snow temperatures ptn.
    DO ji = 1,kjpindex
      IF (SUM(snowdz(ji,:)) .GT. 0) THEN
        snowtemp(ji,1) = (lambda_snow(ji) * cgrnd_snow(ji,1) + (temp_sol_new(ji)+temp_sol_add(ji))) / &
             (lambda_snow(ji) * (un - dgrnd_snow(ji,1)) + un)
        temp_sol_add(ji) = zero 
        DO jg = 1,nsnow-1
            snowtemp(ji,jg+1) = cgrnd_snow(ji,jg) + dgrnd_snow(ji,jg) * snowtemp(ji,jg)
        ENDDO
      ENDIF
    ENDDO

    !yx:+++++
    !PFT-specific calculation
    DO jv = 1,nvm
      DO ji = 1,kjpindex
        IF ( SUM(snowdz_pft(ji,:,jv)) .GT. 0 ) THEN
          IF ( ok_LAIdev_sechiba(jv) ) THEN
              snowtemp_pft(ji,1,jv) = (lambda_snow_pft(ji,jv) * cgrnd_snow_pft(ji,1,jv) + (temp_sol_new_pft(ji,jv)+temp_sol_add_pft(ji,jv))) / &
                   (lambda_snow_pft(ji,jv) * (un - dgrnd_snow_pft(ji,1,jv)) + un)
              temp_sol_add_pft(ji,jv) = zero
              DO jg = 1,nsnow-1
                  snowtemp_pft(ji,jg+1,jv) = cgrnd_snow_pft(ji,jg,jv) + dgrnd_snow_pft(ji,jg,jv) * snowtemp_pft(ji,jg,jv)
              ENDDO
          ELSE
              snowtemp_pft(ji,1,jv) = (lambda_snow_pft(ji,jv) * cgrnd_snow_pft(ji,1,jv) + (temp_sol_new(ji)+temp_sol_add(ji))) / &
                   (lambda_snow_pft(ji,jv) * (un - dgrnd_snow_pft(ji,1,jv)) + un)
              temp_sol_add_pft(ji,jv) = zero
              DO jg = 1,nsnow-1
                  snowtemp_pft(ji,jg+1,jv) = cgrnd_snow(ji,jg) + dgrnd_snow(ji,jg) * snowtemp(ji,jg)
              ENDDO
          ENDIF
        ENDIF
      ENDDO
    ENDDO
    !yx:-----

    IF (printlev>=3) WRITE (numout,*) ' explicitsnow_profile done '

  END SUBROUTINE explicitsnow_profile

END MODULE explicitsnow





